#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : __init__.py
# @Time         : 2023/8/4 10:33
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  : https://github.com/mher/flower/tree/master
"""
flower是基于web的监控和管理Celery的工具，和任务队列是隔离的，flower的运行并不会影响到任务队列的真正执行。
flower作为celery后台任务的管理工具，将各个任务的执行情况、各个worker的健康状态进行实时监控并以可视化的方式展现。
"""
from celery import Celery, shared_task, states

from meutils.pipe import *
from meutils.serving.celery import config


Celery().conf.from_object(config)






#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : test
# @Time         : 2024/10/17 18:45
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  :
import io

from meutils.pipe import *

from PIL import Image, ImageDraw

# data = json.loads(Path('p1.json').read_text())
#
data = json.loads(Path('p2.json').read_text())

from PIL import Image, ImageDraw

def crop_polygon(image_path, outline_points, inline_points):
    if isinstance(image_path, bytes):
        image_path = io.BytesIO(image_path)

    # 打开图像
    img = Image.open(image_path)
    print(img.size)

    # 创建一个与原图大小相同的黑色遮罩
    mask = Image.new('L', img.size, 0)

    # 在遮罩上绘制白色多边形
    for points in outline_points:
        ImageDraw.Draw(mask).polygon(points, outline=1, fill=255)

    for points in inline_points:
        ImageDraw.Draw(mask).polygon(points, outline=1, fill=0)

    # 将遮罩应用到原图
    output = Image.new('RGBA', img.size, (0, 0, 0, 0))
    output.paste(img, (0, 0), mask)

    # # 将 PIL Image 转换为字节
    # buffer = io.BytesIO()
    # output.save(buffer, format="PNG")
    # byte_data = buffer.getvalue()
    #
    # return byte_data
    return output


if __name__ == '__main__':
    # 使用示例
    # image_path = 'img.png'
    image_path = 'test.jpg'
    image_path = open('test.jpg', 'rb').read()

    # points = list(np.array(points))
    n = 3567 / 1280
    n = 2.75
    n = 1

    outline_points = []
    inline_points = []

    for outlines in data['foreground'][0].get('outlines', []):
        _ = list(map(lambda point: (point['x'], point['y']), outlines['line']))
        outline_points.append(_)

    for inlines in data['foreground'][0].get('inlines', []):
        _ = list(map(lambda point: (point['x'], point['y']), inlines['line']))
        inline_points.append(_)

    result = crop_polygon(image_path, outline_points, inline_points)
    result.save('output1111.png')

    type(result)


#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : redis
# @Time         : 2024/3/26 11:21
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  :
# from meutils.pipe import *

import os
from redis import Redis, ConnectionPool
from redis.asyncio import Redis as AsyncRedis, ConnectionPool as AsyncConnectionPool

kwargs = {
    "retry_on_timeout": True,
    # "db": 6
}
if REDIS_URL := os.getenv("REDIS_URL"):
    # logger.debug(REDIS_URL)

    pool = ConnectionPool.from_url(REDIS_URL, **kwargs)
    redis_client = Redis.from_pool(pool)

    async_pool = AsyncConnectionPool.from_url(REDIS_URL, **kwargs)
    redis_aclient = AsyncRedis.from_pool(async_pool)
    # redis_client = Redis.from_url(REDIS_URL, **kwargs)
    # redis_aclient = AsyncRedis.from_url(REDIS_URL, **kwargs)

else:
    redis_client = Redis(**kwargs)  # decode_responses=True
    redis_aclient = AsyncRedis(**kwargs)


async def sadd(name, *values, ttl: int = 0):
    await redis_aclient.sadd(name, *values)

    if ttl:
        await redis_aclient.expire(name, ttl)


if __name__ == '__main__':
    from meutils.pipe import *

    # print(arun(redis_aclient.get("")))
    # print(redis_client.lrange("https://api.moonshot.cn/v1",0, -1))

    # print(redis_client.lrange("https://api.deepseek.com/v1",0, -1))
    # print(redis_client.exists("https://api.deepseek.com/v1"))

    # print(type(redis_aclient.get("test")))

    # print(redis_client.delete("https://api.deepseek.com/v1"))
    feishu_url = "https://xchatllm.feishu.cn/sheets/Bmjtst2f6hfMqFttbhLcdfRJnNf?sheet=79272d"
    feishu_url = "https://xchatllm.feishu.cn/sheets/GYCHsvI4qhnDPNtI4VPcdw2knEd?sheet=EYgZ8c"

    # with timer():
    #     print(feishu_url in redis_client)
    #
    # with timer():
    #     redis_client.exists(feishu_url)
    # with timer():
    #     print(redis_client.llen(feishu_url))

    # print(redis_client.set('a', 'xx21212'))
    # print(redis_client.set('b', b'xx21212'))
    #
    # print(redis_client.get('a'))
    # print(redis_client.get('b'))

    #
    # print(redis_client.type(feishu))
    # _ = redis_client.lrange(feishu, 0, -1)
    # print(len(eval(_)))

    task_id = "celery-task-meta-ca94c602-a2cc-4db5-afe4-763f30df8a18"

    # arun(redis_aclient.get('celery-task-meta-72d59447-1f88-4727-8067-8244c2268faa'))
    #
    # arun(redis_aclient.select(1))

    # async def main():
    #     r = await redis_aclient.select(1)
    #     return await redis_aclient.get(task_id)
    #
    #
    # arun(main())

    # async def main():
    #     return await redis_aclient.lpop("redis_key")

    # arun(main())

    # r = redis_client.sadd('set1', 'a', 'b', 'c')
    # r = redis_client.sadd('set1', 'd')
    # k="meutils.config_utils.lark_utils.commonaget_spreadsheet_values()[(＇feishu_url＇, ＇https://xchatllm.feishu.cn/sheets/GYCHsvI4qhnDPNtI4VPcdw2knEd?sheet=Gvm9dt＇), (＇to_dataframe＇, True)]	"
    # redis_client.delete(k)
    # print(redis_client.get('test'))

    # print(redis_client.delete("k"))

    # print(type(redis_client.type('pods').decode()))
    # print(redis_client.get('xxsadasd').decode())

    # redis_client.set("pods", "10.219.11.231 114.66.55.228")





# !/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : demo
# @Time         : 2024/8/7 09:06
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  :

from meutils.pipe import *
from meutils.schemas.openai_types import CompletionRequest
from meutils.io.files_utils import to_file

from meutils.apis.gradio_api.utils import create_client, handle_file


class Completions(object):

    def __init__(
            self,
            base_url: Optional[str] = None,
            api_key: Optional[str] = None,
    ):

        self.api_key = api_key

    async def create(self, request: CompletionRequest):

        if not (image_urls := request.last_urls.get("image_url")):
            yield "# Image is required"
            return

        if hasattr(request, "task_type"):
            task_type = request.task_type
        else:
            task_type = "Free OCR"

        image_url = image_urls[0]
        if image_url.startswith("http"):
            image = handle_file(image_url)
        else:
            image = await to_file(image_url)
            image = handle_file(image)

        for i in range(2):
            try:

                client = await create_client(request.model, hf_token=self.api_key)

                result = client.predict(
                    image=image,
                    model_size="Gundam (Recommended)",
                    # task_type="Convert to Markdown",
                    # task_type Literal['Free OCR', 'Convert to Markdown'] Default: "Convert to Markdown"
                    # task_type="Free OCR",
                    task_type=task_type,
                    api_name="/process_image",
                )
                # task_type = "📄 Convert to Markdown",
                # ref_text = "Hello!!",
                # api_name = "/process_ocr_task"
                _, _, text = result

                yield text
                break
            except Exception as e:
                logger.error(e)
                # proxy = await get_one_proxy()
                # httpx_kwargs = {
                #     "proxy": proxy,
                # }
                if i == 1:
                    raise e


if __name__ == '__main__':
    model = "khang119966/DeepSeek-OCR-DEMO"
    model = "axiilay/DeepSeek-OCR-Demo"

    request = CompletionRequest(
        model=model,
        messages=[
            {
                "role": "user",
                "content": [
                    {
                        "type": "image_url",
                        "image_url": {
                            "url": "https://s3.ffire.cc/files/pdf_to_markdown.jpg",
                        }
                    }
                ]
            }
        ],
        task_type="Convert to Markdown",
    )

    arun(Completions().create(request))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : common
# @Time         : 2024/11/28 15:28
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  : 

from meutils.pipe import *
from meutils.decorators.retry import retrying

from meutils.async_task import celery_config

from celery import Celery, Task, shared_task, states



worker = Celery()

worker.config_from_object(celery_config)

worker.conf.update(
    # result_expires=30 * 24 * 60 * 60,
    # enable_utc=False,
    # timezone='Asia/Shanghai',
    task_track_started=True,
)

if __name__ == '__main__':
    print(worker.conf.humanize(with_defaults=False))

    print(worker.conf.broker_url)
    print(worker.conf.result_backend)

    print(arun(get_task("59ddf636-3f27-4110-948e-5977c8cbe1b3")))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : luma
# @Time         : 2024/7/22 11:12
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  : 


from meutils.pipe import *
from meutils.decorators.retry import retrying
from meutils.schemas.luma_types import BASE_URL, LumaRequest, EXAMPLES
from meutils.schemas.task_types import Task

from meutils.notice.feishu import send_message as _send_message
from meutils.config_utils.lark_utils import get_next_token_for_polling

FEISHU_URL = "https://xchatllm.feishu.cn/sheets/Bmjtst2f6hfMqFttbhLcdfRJnNf?sheet=RsoADK"

send_message = partial(
    _send_message,
    title=__name__,
    url="https://open.feishu.cn/open-apis/bot/v2/hook/dc1eda96-348e-4cb5-9c7c-2d87d584ca18"
)


def refresh_token():
    import requests

    url = "https://lumalabs.ai/?_rsc=1ovtp"

    payload = {}
    headers = {
        'pragma': 'no-cache',
        'priority': 'u=1, i',
        'Cookie': '_clck=1pg7ir3%7C2%7Cfno%7C0%7C1664; _ga=GA1.1.708578083.1721630345; luma_session=SkMABQUEQBc6TA4PUVoaD0ERUEsvDFFxUFl6Cyl4Zk14UC8PLxJ6CjcNAid9AnFeEWtjcS9dHVNKek8GNHhaeF8SCAI-L18HCQFQAGNYU3wLXFxrMQdKe2R2WDsZaEduZQUKKA9RAykycgwoYFxQbyVQQXwyMll7d2JYLydaSm4DNwwvDxZaPVdAET5jAQhtU19eew8URlJkSEYEMEZabWkJFi8MDksqH3EZKl55QXgLWE9UNEpJBVVUUS5XdV5RegsSAlNWWA5UUVQJRkMVAjd3dnhUVQFsA15xDSUBVkJaQ01GRBNWAhddEgxrQFdeBF0XCEVGVk95WFclAFh8XnsoNBwvUH0NLEsoCmYBW3YoBXxZFTxldXkJGwcae0lTZigIKQgSWgA9dg0HWA0JUTZfXnsPC1pvZ1NMLzR3Xm5LOBU_MgVYKgwIUSljflV5NVtdaCEHR3hkZl8vJ2NeenUKGD9UN14tDE8IPQZMSG82BgVqVwhYf1lAQAY0SUBRYhYIPD4JRC0PVxkqTn1BeAt-BH8PD0lQYh5wDjNXR050BVFfAgJ3MA5sJAZCZ0tWFmV9XQkqC0JRRGM4FWFabn47CFYzQx9ER1sNDVFaTGoIVxcIRQpGWl9N; _clsk=k5j7yi%7C1721631686971%7C7%7C0%7Cw.clarity.ms%2Fcollect; _ga_67JX7C10DX=GS1.1.1721630344.1.1.1721631758.0.0.0',
        'User-Agent': 'Apifox/1.0.0 (https://apifox.com)'
    }

    response = requests.request("GET", url, headers=headers, data=payload)
    #
    # print(response.cookies)
    # print(response.cookies.values())
    # print(response.headers)
    #
    # print(response.text)


@retrying(max_retries=6, predicate=lambda r: not r)
async def create_task(request: Union[dict, LumaRequest], token: Optional[str] = None):
    token = token or await get_next_token_for_polling(FEISHU_URL)
    logger.debug(token)

    headers = {
        'Cookie': token,
    }

    payload = request if isinstance(request, dict) else request.model_dump(exclude_none=True)

    async with httpx.AsyncClient(base_url=BASE_URL, headers=headers, timeout=30) as client:
        response = await client.post("/generations/", json=payload)
        # [
        #     {
        #         "id": "82507194-91c0-4fe3-ad12-f459acbcf3a5",
        #         "prompt": "清凉夏季美少女，微卷短发，运动服，林间石板路，斑驳光影，超级真实，16K",
        #         "state": "pending",
        #         "created_at": "2024-07-22T05:26:20.871974Z",
        #         "video": null,
        #         "liked": null,
        #         "estimate_wait_seconds": null,
        #         "thumbnail": null,
        #         "last_frame": null
        #     }
        # ]

        logger.debug(response.text)
        logger.debug(response.status_code)

        if response.is_success:
            data = response.json()
            send_message(bjson(data))
            data = data[0]

            return Task(id=data.get("id"), data=data, system_fingerprint=token)

        response.raise_for_status()


# @retrying(predicate=lambda r: not r)  # 触发重试
# @retrying()  # 触发重试
async def get_task(task_id: str, token: str):  # 1ee02f7b-42df-4926-b034-f56e4c4e2d31

    headers = {
        'Cookie': token,
    }
    params = {
        'offset': 0,
        'limit': 32
    }
    async with httpx.AsyncClient(base_url=BASE_URL, headers=headers, timeout=30) as client:
        # response = await client.get(f"/user/generations/", params=params)
        # if response.is_success:
        #     data = response.json()
        #     return list(filter(lambda x: x.get("id") == task_id, data))
        response = await client.get(f"/generations/{task_id}", params=params)

        logger.debug(response.text)
        logger.debug(response.status_code)
        if response.is_success:
            return response.json()

        response.raise_for_status()


if __name__ == '__main__':
    pass
    # cookie = "_clck=1tbe3d1%7C2%7Cfno%7C0%7C1664; _ga=GA1.1.1897085930.1721613998; luma_session=SkMABQUEQBc6TA4PUVoaD0ERUEsvDFFxUFl6Cyl4Zk14UC8PLxJ6CjcNAid9AnFeEWtjcS9dHVNKek8GNHhaeF8SCAI-L18HCQFQAGNYU3wLXFxrMQdKe2R2WDsZaEduZQUKKA9RAykycgwoYFxQbyVQQXwyMll7d2JYLydaSm4DNwwvDxZaPVdAET5jAQhtU19eew8URlJkSEYEMEZabWkJFi8MDksqH3EZKl59C3glWExUNEpccnsHWS8wB3lffhIoLQRTWDYDdwpVGXJ8TS93b3QREFlGRARmASxCAmEFQ01GRBNWAhddEgxrQFdeBF0XCEVGVk95WFclAFh8XnsoNBwvUH0NLEsoCmYBW3YoBXxZFTxldXkJGwcae0lTZigIKQgSWgA9dg0HWA0JUTZfXnsPC1pvZ1NMLzR3Xm5LOBU_MgVYKgwIUSljflV5NVtdaCEHR3hkZl8vJ2NeenUKGD9UN14tDE8IPQZMSG82BgVqVwhYf1lAQAY0SUBRYhYIPD4JRC0PVxkqTn1BeAt-Qn8hD0pQYh5tVTtZYlIDDjlTKAVhNgxJJzMBXwENUB5AcC8NYH1-YQdbAAN9TWMbGAUNQx9ER1sNDVFaTGoIVxcIRQpGWl9N"
    # cookie = "_clck=1tbe3d1%7C2%7Cfno%7C0%7C1664; _ga=GA1.1.1897085930.1721613998; luma_session=SkMABQUEQBc6TA4PUVoaD0ERUEsvDFFxUFl6Cyl4Zk14UC8PLxJ6CjcNAid9AnFeEWtjcS9dHVNKek8GNHhaeF8SCAI-L18HCQFQAGNYU3wLXFxrMQdKe2R2WDsZaEduZQUKKA9RAykycgwoYFxQbyVQQXwyMll7d2JYLydaSm4DNwwvDxZaPVdAET5jAQhtU19eew8URlJkSEYEMEZabWkJFi8MDksqH3EZKl59DHo1dkxUNEpqZVJldgA1WmZ8QxUAIycRchwHcCUmfwEJAwBnAgM1MFp_QXFjKFpHSQV8Q01GRBNWAhddEgxrQFdeBF0XCEVGVk95WFclAFh8XnsoNBwvUH0NLEsoCmYBW3YoBXxZFTxldXkJGwcae0lTZigIKQgSWgA9dg0HWA0JUTZfXnsPC1pvZ1NMLzR3Xm5LOBU_MgVYKgwIUSljflV5NVtdaCEHR3hkZl8vJ2NeenUKGD9UN14tDE8IPQZMSG82BgVqVwhYf1lAQAY0SUBRYhYIPD4JRC0PVxkqTn1BeAt-TX0xIUpQYh5TByJhAn5wODYsCTIHLhB6DFZNUWsYE3sAZB0gBA9ydVInDFZyRGYAUgIrQx9ER1sNDVFaTGoIVxcIRQpGWl9N"
    arun(create_task(EXAMPLES[0]))
    # arun(get_task("6de49fdc-f3ed-4590-8a1a-0c53cfca055b", cookie))
    # print(refresh_token())



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : kolors
# @Time         : 2024/7/25 08:42
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  : https://huggingface.co/spaces/gokaygokay/KolorsPlusPlus
# https://huggingface.co/spaces/Kwai-Kolors/Kolors-Virtual-Try-On

from meutils.pipe import *
from meutils.decorators.retry import retrying
from meutils.config_utils.lark_utils import get_next_token_for_polling

from meutils.apis.hf.gradio import create_client, handle_file
from meutils.io.files_utils import to_file

FEISHU_URL_HF = "https://xchatllm.feishu.cn/sheets/MekfsfVuohfUf1tsWV0cCvTmn3c?sheet=NlJ2h0"
FEISHU_URL_MODELSCOPE = "https://xchatllm.feishu.cn/sheets/MekfsfVuohfUf1tsWV0cCvTmn3c?sheet=305f17"

CLOTH_IMAGES = [
    "https://oss.ffire.cc/files/x.png",
    "https://s5k.cn/api/v1/studio/Kwai-Kolors/Kolors-Virtual-Try-On/gradio/file=/tmp/gradio/9d1842bb0c1f9dfa52696e3d8a63fa006e158f00/garment1.png",
    "https://s5k.cn/api/v1/studio/Kwai-Kolors/Kolors-Virtual-Try-On/gradio/file=/tmp/gradio/6349e792201f972c10904fa71cef3559bae35b28/10_dress.png"
]


class KolorsTryOnRequest(BaseModel):
    model: str = "kolors-virtual-try-on"
    human_image: str = "https://oss.ffire.cc/files/try-on.png"
    cloth_image: str = Field(default_factory=lambda: np.random.choice(CLOTH_IMAGES))
    seed: int = 0

    def __init__(self, /, **data: Any):
        super().__init__(**data)
        if self.cloth_image == self.human_image:
            self.cloth_image = np.random.choice(CLOTH_IMAGES)


@retrying(max_retries=2, min=3, predicate=lambda r: not r["data"])
async def create(request: KolorsTryOnRequest, endpoint: Optional[str] = None, token: Optional[str] = None):
    logger.debug(request)

    endpoint = endpoint or "https://s5k.cn/api/v1/studio/Kwai-Kolors/Kolors-Virtual-Try-On/gradio/"
    if endpoint.startswith('https://s5k.cn'):
        FEISHU_URL = FEISHU_URL_MODELSCOPE
    else:
        FEISHU_URL = FEISHU_URL_HF

    token = token or await get_next_token_for_polling(FEISHU_URL, from_redis=True)
    logger.info(token)

    try:
        client = await create_client(endpoint, hf_token=token)

        human_image, cloth_image = map(
            handle_file,
            await asyncio.gather(*map(to_file, [request.human_image, request.cloth_image]))
        )
        result = client.predict(
            # person_img=handle_file("tryon.png"),

            person_img=human_image,  # 有oss限制
            garment_img=cloth_image,
            seed=request.seed,
            randomize_seed=True,
            api_name="/tryon"
        )

        logger.debug(result)

        url = result[0]["url"]
        return {
            "data": [{"url": url}]
        }

    except Exception as e:
        logger.error(e)
        return {
            "data": [],
        }


if __name__ == '__main__':
    token = "e3f5804a-869e-460c-9470-946be0624963"
    request = KolorsTryOnRequest(
        # human_image="https://oss.ffire.cc/files/try-on.png",
        # person_image="https://oss.ffire.cc/files/kling_watermark.png",
        # cloth_image='xx'
    )

    print(request)
    with timer():
        arun(create(request))

    # from gradio_client import Client, handle_file
    #
    # client = Client("https://s5k.cn/api/v1/studio/Kwai-Kolors/Kolors-Virtual-Try-On/gradio/", hf_token=token, download_files=False)
    # result = client.predict(
    #     person_img=handle_file(request.person_image),
    #     garment_img=handle_file(request.garment_image),
    #     seed=0,
    #     randomize_seed=True,
    #     api_name="/tryon",
    # )
    # print(result)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : test
# @Time         : 2023/11/17 15:24
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  : 

from meutils.pipe import *

scene_prompt = """
You are an expert in understanding scene transitions based on visual features in a video. For the given sequence of images per timestamp, identify different scenes in the video. Generate audio description for each scene with time ranges.
Be sure to answer in Chinese.
""".strip()

audio_prompt = """
You are an expert at understanding audio descriptions of different scenes in a video. Can you leverage the information provided, including title, abstract, audio descriptions and generate full audio description of each scene with non overlapping time ranges. Keep as many scenes possible covering all time ranges. You may find character names in the title or abstract. Use character names wherever possible in the audio descriptions. Keep the audio description for each time range within one short sentence.
Be sure to answer in Chinese.
""".strip()

messages = [
    # {
    #     "role": "system",
    #     "content": [
    #         {"type": "text", "text": "What’s in this image?"},
    #     ]
    # },
    {
        "role": "user",
        "content": [
            {"type": "text", "text": scene_prompt},
            {
                "type": "image_url",
                "image_url": {
                    "url": "https://upload.wikimedia.org/wikipedia/commons/thumb/d/dd/Gfp-wisconsin-madison-the-nature-boardwalk.jpg/2560px-Gfp-wisconsin-madison-the-nature-boardwalk.jpg",
                    "detail": "auto"
                },
            },
        ],
    }
]



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : tin
# @Time         : 2025/7/8 13:09
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  : 

from meutils.pipe import *
from tinytag import TinyTag
import requests
from io import BytesIO

url = "https://lmdbk.com/5.mp4"
# url = "https://ark-content-generation-cn-beijing.tos-cn-beijing.volces.com/doubao-seedance-1-0-pro/02175195186682100000000000000000000ffffac15f035b51e5d.mp4?X-Tos-Algorithm=TOS4-HMAC-SHA256&X-Tos-Credential=AKLTYjg3ZjNlOGM0YzQyNGE1MmI2MDFiOTM3Y2IwMTY3OTE%2F20250708%2Fcn-beijing%2Ftos%2Frequest&X-Tos-Date=20250708T051814Z&X-Tos-Expires=86400&X-Tos-Signature=7c8be9f1694e7a437bd7263ad7ddee42c642de190c1919f0bd776e6416cb1772&X-Tos-SignedHeaders=host"
# 下载文件到内存
headers = {
    'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/91.0.4472.124 Safari/537.36',
    'Range': 'bytes=0-8191'  # 请求文件的前 8192 字节 (8KB)，通常足够了
}


# # 发起流式请求，只获取部分内容
# response = requests.get(url, headers=headers, stream=False, timeout=10)
# response.raise_for_status()  # 如果请求失败 (如 404), 会抛出异常
# # 读取响应的前 8KB 内容
# video_data = response.content
#
# len(response.content)
#
#
#
#
# response = requests.get(url, stream=True)
# audio_data = BytesIO(response.content)

# 解析时长
# tag = TinyTag.get(filename='.mp4', file_obj=audio_data)
# duration = tag.duration  # 单位：秒
# print(f"音频时长：{duration:.2f}秒")

# filename = "/Users/betterme/PycharmProjects/AI/MeUtils/meutils/ai_audio/y5-1YTGpun17eSeggZMzX_video-1733468228.mp4"
#
# # 解析时长
# tag = TinyTag.get(filename)
# duration = tag.duration  # 单位：秒
# print(f"音频时长：{duration:.2f}秒")

#
async def get_duration(url, filename: str = ".mp4"):
    # Path(url.split('?')[0]).name
    headers = {
        "Range": "bytes=0-8191"
    }
    async with httpx.AsyncClient(headers=headers, timeout=100) as client:
        response = await client.get(url=url)
        response.raise_for_status()

        chunk = io.BytesIO(response.content)
        tag = TinyTag.get(filename=filename, file_obj=chunk, ignore_errors=False)
        return np.ceil(tag.duration or 10)


if __name__ == '__main__':
    with timer():
        arun(get_duration(url))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : MeUtils.
# @File         : sftp
# @Time         : 2022/9/26 上午11:19
# @Author       : yuanjie
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  :  sftp -P 57891 comp_5153@222.76.203.180


import paramiko


class SFTPClient(object):

    def __init__(self, host='222.76.203.180', port=57891, username='comp_5153', password='7NmIt00rGF5vKGlt'):
        sf = paramiko.Transport((host, port))
        sf.connect(username=username, password=password)
        self.sftp = paramiko.SFTPClient.from_transport(sf)

    def put(self, localpath, remotepath):
        self.sftp.put(localpath, remotepath)


def submit(localpath, remotepath, username, password, sock):
    import paramiko

    with paramiko.Transport(sock) as sf:
        sf.connect(username=username, password=password)

        sftp = paramiko.SFTPClient.from_transport(sf)
        sftp.put(localpath, remotepath)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : utils
# @Time         : 2024/6/20 09:08
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  :


from meutils.pipe import *
from meutils.schemas.openai_types import CompletionRequest


def oneturn2multiturn(messages: Union[List[dict], CompletionRequest], template: Optional[str] = None,
                      ignore_system: bool = True):
    """todo: https://github.com/hiyouga/LLaMA-Factory/blob/e898fabbe3efcd8b44d0e119e7afaed4542a9f39/src/llmtuner/data/template.py#L423-L427

    _register_template(
    name="qwen",
    format_user=StringFormatter(slots=["<|im_start|>user\n{{content}}<|im_end|>\n<|im_start|>assistant\n"]),
    format_system=StringFormatter(slots=["<|im_start|>system\n{{content}}<|im_end|>\n"]),
    format_observation=StringFormatter(slots=["<|im_start|>tool\n{{content}}<|im_end|>\n<|im_start|>assistant\n"]),
    format_separator=EmptyFormatter(slots=["\n"]),
    default_system="You are a helpful assistant.",
    stop_words=["<|im_end|>"],
    replace_eos=True,
)
    :return:
    """
    # from jinja2 import Template, Environment, PackageLoader, FileSystemLoader
    #
    # system_template = Template("<|im_start|>system\n{{content}}<|im_end|>\n")  # .render(content='xxxx')
    # user_template = Template("<|im_start|>user\n{{content}}<|im_end|>\n")  # 最后<|im_start|>assistant\n
    # assistant_template = Template("<|im_start|>assistant\n{{content}}<|im_end|>\n")

    # todo: [{"type": "image_url", "image_url": {"url": ""}}]] 单独处理
    # 混元不是很感冒
    # context = "\n"
    # for message in messages:
    #     role, content = message.get("role"), message.get("content")
    #     context += f"<|im_start|>{role}\n{content}<|im_end|>\n"
    # context += "<|im_start|>assistant\n"
    if isinstance(messages, CompletionRequest):
        messages = messages.messages

    if len(messages) == 1:
        content = messages[0].get("content")
        if isinstance(content, list):
            for c in content:
                if c.get("type") == "text":
                    content = c.get("text", "")
                    break
        return content

    context = "\n"
    for message in messages:
        role = message.get("role")
        content = message.get("content", "")

        if role == "system" and ignore_system:
            continue

        if isinstance(content, list):  # content: {'type': 'text', 'text': ''}
            for c in content:
                if c.get("type") == "text":
                    content = c.get("text", "")
                    break

        context += f"{role}:\n{content}\n\n"

        # logger.debug(context)

    return context


if __name__ == '__main__':
    content = [
        {
            "type": "text",
            "text": """
            # 身份
    你是一个高精度的【视频内容转录员】。

    # 核心任务
    你的唯一任务是：以【时间戳】为单位，逐字逐句、逐个动作地记录视频中的所有视听信息。你必须像法庭书记员一样，做到100%的客观和精确。

    # 输出格式 (必须严格遵守)
    使用以下格式，为每个独立事件创建新的一行：
    [HH:MM:SS] [类别]: [内容]

    # “类别”包括：
    - **[台词]**: 记录人物说的每一句话。如果能识别说话人，格式为 `[台词-人名]`. 如果不能，格式为 `[台词-男/女]`.
    - **[画面]**: 描述场景、环境、人物外貌、表情和关键物品。
    - **[动作]**: 描述人物的所有具体动作。
    - **[音效]**: 记录所有非对话的声音，如(敲门声), (汽车鸣笛)。

    # 绝对禁止
    - 禁止进行任何总结、分析或评论。
    - 禁止使用任何剧本格式。
    - 禁止省略任何细节。
    - 你的所有输出都必须是简体中文。
            """
        },
        {"type": "video_url", "video_url": {
            "url": "https://qwen-webui-prod.oss-accelerate.aliyuncs.com/310cbdaf-3754-461c-a3ff-9ec8005329c9/b813de59-c7d6-4c58-9f0d-0a8a283ca41b_1%20%282%29.mp4?x-oss-security-token=CAISzwJ1q6Ft5B2yfSjIr5nsCI7Qt5tAhKunaFGDilI6QLpppIPvoDz2IHhMenlvAewetvg2nGBT7fkflrN6SJtIXleCZtF94plR7QKoZ73Zocur7LAJksVu3r1e%2F0WpsvXJasDVEfn%2FGJ70GX2m%2BwZ3xbzlD0bAO3WuLZyOj7N%2Bc90TRXPWRDFaBdBQVGAAwY1gQhm3D%2Fu2NQPwiWf9FVdhvhEG6Vly8qOi2MaRmHG85R%2FYsrZN%2BNmgecP%2FNpE3bMwiCYyPsbYoJvab4kl58ANX8ap6tqtA9Arcs8uVa1sruE3ebbqLqoE3dFMgPvhhQPEZtpj6krhxuanUj5Q8SO%2FTtljJOs62Z%2FdDoKOscIvBXr6yRaJvreicfPqb1%2FQnHpLpLAPa1a4HqTQbIAuUI3bCJL8U1AEJVr6kzHkNhYYrssUUla4R5TGOWbJLJyr%2BHN3xuRqAAZjIuAflHa2njQWXjxvTYM83wepsCAQnJw79bVYI%2FDbZWdln2va6OOjJNxwvdbV42mujTd5qofC%2Bb39J0q5fX6w%2FRanfViZwgN0Bd9tLLLzrjMl02ZLDtq14O5yVY2Z0qyJGw7AAy01Hsjn8BmYaWX19LEfLWe6hrq%2FjSnA9JCUfIAA%3D&x-oss-date=20251008T091359Z&x-oss-expires=300&x-oss-signature-version=OSS4-HMAC-SHA256&x-oss-credential=STS.NZYC4dZDa3iLjw2nRoL5EKFDB%2F20251008%2Fap-southeast-1%2Foss%2Faliyun_v4_request&x-oss-signature=96ae6ce2d0e12e73cc709a1855b772ca2dc9f63be10399fbcdfd4d63f8e53e8d"}}
    ]
    messages = [
        {
            "role": "system",
            "content": [
                {
                    "type": "text",
                    "text": "你是数学家"
                }
            ]
        },
        {
            "role": "user",
            "content": [
                {
                    "type": "text",
                    "text": "1+1"
                }
            ]
        },
        {
            "role": "assistant",
            "content": [
                {
                    "type": "text",
                    "text": "2"
                }
            ]
        },

        {
            "role": "user",
            "content": content
        },

    ]

    # print(oneturn2multiturn(messages, ignore_system=False))
    print(oneturn2multiturn(messages, ignore_system=True))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : meta
# @Time         : 2024/11/11 17:03
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  : todo 重构

from meutils.pipe import *
from meutils.schemas.metaso_types import FEISHU_URL, BASE_URL, MetasoRequest, MetasoResponse
from meutils.decorators.retry import retrying
from meutils.config_utils.lark_utils import get_next_token_for_polling
from meutils.apis.proxy.ips import FEISHU_URL_METASO, get_one_proxy, get_proxies
from meutils.schemas.openai_types import ChatCompletionRequest, CompletionRequest
from meutils.notice.feishu import send_message

from urllib.parse import quote_plus, unquote_plus

token = "wr8+pHu3KYryzz0O2MaBSNUZbVLjLUYC1FR4sKqSW0p19vmcZAoEmHC72zPh/fHtOhYhCcR5GKXrxQs9QjN6dulxfOKfQkLdVkLMahMclPPjNVCPE8bLQut3zBABECLaSqpI0fVWBrdbJptnhASrSw=="

MODELS = {

    "ai-search": "detail",
    "ai-search-pro": "research",

    "ai-search:scholar": "detail",
    "ai-search-pro:scholar": "research",

    "deepseek-r1-metasearch": "strong-research",

    "deepseek-r1-metaresearch": "strong-research",

    "meta-research": "strong-research",

    "meta-search": "detail",
    "meta-deepsearch": "strong-research",
    "meta-deepresearch": "strong-research",

    "meta-search:scholar": "detail",
    "meta-deepsearch:scholar": "strong-research",
    "meta-deepresearch:scholar": "strong-research",

}

# pattern = re.compile('\[\[(\d+)\]\]')
pattern = re.compile(r'\[\[(\d+)\]\]')


def replace_ref(match):
    ref_num = match.group(1)
    return f'[^{ref_num}]'  # [[1]] -> [^1]


async def get_session_id(request: MetasoRequest, headers: Optional[dict] = None, proxy: Optional[str] = None):
    if proxy:
        logger.debug(proxy)

    headers = headers or {
        # "cookie": "uid=65fe812a09727c19a54b0328; sid=eb6f4fe9034b4c9497fceca7ff6bafdd",
        # "cookie": f"uid={shortuuid.random(24).lower()}; sid={shortuuid.random(32).lower()}"

        # "cookie": "JSESSIONID=8B267A5E2299C2BF46EB354104AACF76; tid=b103b947-be89-40b8-b162-80fdcda60807; aliyungf_tc=8cfbed3e5fd53c2a81605c7dcb63d45d3114b94750922ef63e04cb5a6ceccba1; s=bdpc; traceid=d21e97f303b546c0; hideLeftMenu=1; usermaven_id_UMO2dYNwFz=1y1t3p1t2t; uid=65fe812a09727c19a54b0328; sid=eb6f4fe9034b4c9497fceca7ff6bafdd; newSearch=false"

        # "token": "wr8+pHu3KYryzz0O2MaBSNUZbVLjLUYC1FR4sKqSW0p19vmcZAoEmHC72zPh/fHtxcdjbUPQpQ+cHJxaEajSgJMlmjIlIUew+aPZMEcnIqI1j3rHg9aAsbcYX/MF8lyJ+zJimUWQ2SOBo4yJQ6yUOQ=="
    }

    payload = request.model_dump(exclude_none=True)
    async with httpx.AsyncClient(base_url=BASE_URL, headers=headers, timeout=30, proxy=proxy) as client:
        response = await client.post("/api/session", json=payload)
        response.raise_for_status()
        data = response.json()

        logger.debug(bjson(data))
        # {
        #     "errCode": 4001,
        #     "errMsg": "搜索次数超出限制"
        # }
        if data.get("errCode", 0) == 4001:
            logger.debug(data)
            # request_kwargs = {
            #     "proxies": await get_one_proxy(headers.get("cookie"), exclude_ips="154.40.54.76154.12.35.201"),
            #     # "proxies": proxies,
            # }
            # continue

        return data.get("data", {}).get("id")


@alru_cache(ttl=5 * 60)
@retrying(min=3, predicate=lambda r: r is False)
async def get_access_token(session_id: Optional[str] = None):
    pattern = r'<meta\s+id="meta-token"\s+content="([^"]+)"\s*/>'
    async with httpx.AsyncClient(base_url=BASE_URL, timeout=30) as client:  # 测试token过期时间
        response = await client.get(f"/search/{session_id}")
        response.raise_for_status()

        tokens = re.findall(pattern, response.text)
        if not tokens:
            send_message(response.text, __name__)

        return tokens and tokens[0]


async def create(request: Union[ChatCompletionRequest, CompletionRequest]):
    if isinstance(request, CompletionRequest):
        request = ChatCompletionRequest(**request.model_dump())

    if request.last_content == 'ping':
        yield "pong"
        return

    system_fingerprint = request.system_fingerprint

    engine_type = None
    if ":" in request.model:
        _, engine_type = request.model.split(':')

    model = None
    if any(i in request.model for i in {"deep", "thinking"}):
        model = system_fingerprint = "fast_thinking"

    request = MetasoRequest(
        model=model,
        mode=MODELS.get(request.model, "detail"),
        engineType=engine_type,
        question=request.last_content,
    )

    logger.debug(request.model_dump_json(indent=4))

    headers = {}
    if "research" in request.mode or model=="fast_thinking":  # 登录
        cookie = await get_next_token_for_polling(FEISHU_URL)
        headers["cookie"] = cookie
        logger.debug(cookie)

    proxy = None
    # proxies = await get_proxies()
    session_id = await get_session_id(request, headers=headers, proxy=proxy)
    # session_id = None
    if session_id is None:  # 走代理: 随机轮询

        proxy = await get_one_proxy(feishu_url=FEISHU_URL_METASO)
        session_id = await get_session_id(request, headers=headers, proxy=proxy)

    token = await get_access_token(session_id)

    params = request.model_dump(exclude_none=True)
    params['token'] = token

    async with httpx.AsyncClient(base_url=BASE_URL, headers=headers, params=params, proxy=proxy, timeout=100) as client:
        async with client.stream(method="GET", url="/api/searchV2") as response:
            response.raise_for_status()

            references = []
            async for chunk in response.aiter_lines():

                if (chunk := chunk.strip()) and chunk != "data:[DONE]":
                    # logger.debug(chunk)

                    try:
                        response = MetasoResponse(chunk=chunk)
                        references += response.references

                        if len(response.content) == 1 and response.content.startswith('秘'):  # 替换 模型水印
                            response.content = f"{system_fingerprint} AI搜索，它是一款能够深入理解您的问题的AI搜索引擎。"
                            yield response.content
                            break

                        _ = pattern.sub(replace_ref, response.content)
                        yield _

                    except Exception as e:
                        logger.error(e)
                        logger.debug(response)
            if references:
                for i, ref in enumerate(references, 1):
                    title = ref.get("title")
                    url = ref.get("link") or ref.get("url") or ref.get("file_meta", {}).get("url", "")
                    # logger.debug(url)
                    # url = quote_plus(url)

                    yield f"\n[^{i}]: [{title}]({url})\n"

                # logger.debug(bjson(references))


if __name__ == '__main__':
    # request = MetasoRequest(question="东北证券", mode='research')
    # request = MetasoRequest(question="你是谁", mode='detail', return_raw=True)  # concise
    # request = MetasoRequest(question="Chatfire", mode='concise', return_raw=False)  # concise

    # request = MetasoRequest(question="Chatfire", mode='detail', return_raw=False)  # concise
    # request = MetasoRequest(question="Chatfire", mode='research', response_format=False)  # concise

    # arun(get_session_id(request))
    # arun(get_access_token(request))
    """
    metasearch-
    
    model-mode
    """

    request = ChatCompletionRequest(

        # model="meta-search",
        # model="meta-deepsearch",
        model="meta-deepresearch",

        # model="meta-search:video",

        # model="deepseek-r1-metasearch",

        # model="deepseek-search",
        # model="deepseek-r1-search",
        # model="11meta-deepresearch",

        # model="ai-search",
        # model="ai-search:scholar",
        # model="ai-search-pro:scholar",

        # model="search-pro",

        # messages=[{'role': 'user', 'content': '今天南京天气怎么样'}]
        # messages=[{'role': 'user', 'content': '1+1'}]
        messages=[{'role': 'user', 'content': '周杰伦是谁'}]

    )

    arun(create(request))

    url = "https://www.163.com/v/video/VQAIQ7LPJ.html"

    # print(unquote_plus(url))

    # with timer():
    #     request = MetasoRequest(
    #         model='ds-r1',
    #         mode="research",
    #         question="南京今天天气",
    #
    #     )
    #     arun(get_session_id(request))
    # session_id = "8544840144331366400"
    #
    # arun(get_access_token(session_id))

    # wr8+pHu3KYryzz0O2MaBSNUZbVLjLUYC1FR4sKqSW0p19vmcZAoEmHC72zPh/fHtOhYhCcR5GKXrxQs9QjN6dulxfOKfQkLdVkLMahMclPPjNVCPE8bLQut3zBABECLaSqpI0fVWBrdbJptnhASrSw==



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : models
# @Time         : 2024/11/20 10:57
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  : 

from meutils.pipe import *

from sqlalchemy import JSON
from sqlmodel import Field, Session, SQLModel, create_engine, select, insert, update, Column, DateTime, func


def get_default_name():
    return "xx"


class Hero(SQLModel, table=True):
    id: Optional[int] = Field(default=None, primary_key=True)
    name: str = Field(default_factory=lambda: "test")
    secret_name: str = ""
    age: Optional[int] = None
    # created_at: datetime.datetime = Field(
    #     default_factory=datetime.datetime.utcnow,
    # )
    updated_at: Optional[datetime.datetime] = Field(
        sa_column=Column(DateTime(), onupdate=func.now())
    )


class OneapiTask(SQLModel, table=True):
    """https://github.com/Calcium-Ion/new-api/blob/main/model/task.go"""

    __tablename__ = "tasks"  # 自定义表名

    id: Optional[int] = Field(default=None, primary_key=True)

    task_id: Optional[str] = Field(default=None, max_length=50)
    remote_task_id: Optional[str] = Field(default=None, max_length=50)
    user_id: Optional[int] = Field(default=None)
    channel_id: Optional[int] = Field(default=None)

    data: Optional[dict] = Field(default=None, sa_type=JSON)

    """TaskStatus
    TaskStatusNotStart              = "NOT_START"
    TaskStatusSubmitted             = "SUBMITTED"
    TaskStatusQueued                = "QUEUED"
    TaskStatusInProgress            = "IN_PROGRESS"
    TaskStatusFailure               = "FAILURE"  # todo: 补偿积分+状态记录
    TaskStatusSuccess               = "SUCCESS"
    TaskStatusUnknown               = "UNKNOWN
    """
    status: Optional[str] = Field(default="SUBMITTED", max_length=20)

    progress: str = Field(default="0%", max_length=20)
    fail_reason: Optional[str] = Field(default=None)

    platform: Optional[str] = Field(default="🔥", max_length=30)
    action: Optional[str] = Field(default=None, max_length=40)  # black-forest-labs/flux-schnell

    """价格 50000=0.1 """
    quota: Optional[int] = Field(default=None)  #
    properties: Optional[dict] = Field(default=None, sa_type=JSON)

    submit_time: Optional[int] = Field(default_factory=lambda: int(time.time()))
    start_time: Optional[int] = Field(default_factory=lambda: int(time.time()))
    finish_time: Optional[int] = Field(default_factory=lambda: int(time.time()))

    created_at: Optional[int] = Field(default_factory=lambda: int(time.time()))
    updated_at: Optional[int] = Field(default_factory=lambda: int(time.time()))

    class Config:
        arbitrary_types_allowed = True


class OneapiUser(SQLModel, table=True):
    __tablename__ = "users"  # 自定义表名

    id: Optional[int] = Field(default=None, primary_key=True)
    username: Optional[str] = Field(default=None)
    email: Optional[str] = Field(default=None)

    quota: Optional[int] = Field(default=None)
    used_quota: Optional[int] = Field(default=None)

    request_count: Optional[int] = Field(default=None)

    access_token: Optional[str] = Field(default=None)
    password: Optional[str] = Field(default="")

    class Config:
        arbitrary_types_allowed = True


class OneapiToken(SQLModel, table=True):
    __tablename__ = "tokens"  # 自定义表名

    id: Optional[int] = Field(default=None, primary_key=True)
    user_id: Optional[int] = Field(default=None)
    key: Optional[str] = Field(default=None)

    used_quota: Optional[int] = Field(default=None)
    remain_quota: Optional[int] = Field(default=None)
    unlimited_quota: Optional[bool] = Field(default=False)

    class Config:
        arbitrary_types_allowed = True


class OneapiChannel(SQLModel, table=True):
    __tablename__ = "channels"  # 自定义表名

    id: Optional[int] = Field(default=None, primary_key=True)
    key: Optional[str] = Field(default=None)

    status: Optional[int] = Field(default=None)

    class Config:
        arbitrary_types_allowed = True


if __name__ == '__main__':
    print(Hero())
    # print(Tasks())

    print(OneapiTask.__name__)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : zhipu_智能体
# @Time         : 2024/12/30 17:34
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  : https://bigmodel.cn/dev/api/intelligent-agent-model/assistantapi

from meutils.pipe import *
from meutils.llm.openai_utils import to_openai_params
from meutils.llm.clients import AsyncOpenAI, zhipuai_sdk_client
from meutils.str_utils.regular_expression import parse_url

from meutils.schemas.openai_types import chat_completion, chat_completion_chunk, CompletionRequest, CompletionUsage


class Completions(object):

    def __init__(self,
                 base_url: Optional[str] = None,
                 api_key: Optional[str] = None
                 ):
        pass

    async def create(self, request: CompletionRequest):
        # attachments = [{"file_id": "chatglm4/3db10f79-a952-4987-83d2-cf0cfd5d5530.pdf"}]

        chunks = zhipuai_sdk_client.assistant.conversation(
            assistant_id=request.model,

            model="glm-4-assistant",
            messages=request.messages,
            stream=True,
            attachments=None,
            metadata=None
        )

        references = {}
        for chunk in chunks:
            # logger.debug(chunk)
            delta = chunk.choices[0].delta

            if hasattr(delta, "tool_calls") and delta.tool_calls:
                logger.debug(delta.tool_calls)

                tool_call = delta.tool_calls[0].model_dump()
                tool_type = tool_call.get("type", "")  # drawing_tool code_interpreter web_browser retrieval
                outputs = tool_call.get(tool_type, {}).get("outputs") or []
                # references += outputs

                references.setdefault(tool_type, []).extend(outputs)

                if not outputs:
                    yield f"\n\n```json\n{delta.model_dump_json(indent=4, exclude_none=True)}\n```\n\n"
                logger.debug(references)

                continue

            if hasattr(delta, "content"):
                if references:
                    for i in self.references2content(references):
                        yield i
                    references = {}
                # logger.debug(delta.content)
                yield delta.content

            else:
                logger.debug(delta)

            # if references: # 搜索
            #     urls = [f"[^{i}]: [{ref['title']}]({ref['link']})\n" for i, ref in enumerate(references, 1)]
            #
            #     for url in urls:
            #         yield url
            #
            #     references = []
            #
            # delta = chat_completion_chunk.choices[0].delta.model_construct(**delta.model_dump())
            # chat_completion_chunk.choices[0].delta = delta
            # yield chat_completion_chunk

    def references2content(self, references):
        """drawing_tool code_interpreter web_browser retrieval"""
        if outputs := references.get("web_browser"):
            for i, ref in enumerate(outputs, 1):
                # yield f"[^{i}]: [{ref['title']}]({ref['link']})\n"
                yield f"[{ref['title']}]({ref['link']})\n"

        elif outputs := references.get("drawing_tool"):
            for i, ref in enumerate(outputs, 1):
                yield f"![{i}]({ref['image']})\n"

        yield "\n"

    async def _create(self, request: CompletionRequest):
        if request.last_user_content.startswith(("http",)):
            file_url, text = request.last_user_content.split(maxsplit=1)

            request.messages = [
                {
                    'role': 'user',
                    'content': [
                        {
                            "type": "text",
                            "text": text
                        },

                        {
                            "type": "file",  # 不标准
                            "file_url": {
                                "url": file_url
                            }
                        }
                    ]
                }
            ]

        logger.debug(request)

        data = to_openai_params(request)
        return await self.client.chat.completions.create(**data)


if __name__ == '__main__':
    assistant_id = "65940acff94777010aa6b796"
    # assistant_id = "65d2f07bb2c10188f885bd89"
    # assistant_id="65a265419d72d299a9230616",
    # assistant_id="659d051a5f14eb8ce1235b96",
    # assistant_id="65d2f07bb2c10188f885bd89",

    # assistant_id="659e54b1b8006379b4b2abd6",
    # conversation_id=None,  # 多轮：从messages获取
    # conversation_id="67dd1317d7c8fe4c9efe459a",
    request = CompletionRequest(
        model=assistant_id,
        messages=[
            {
                "role": "user",
                "content": [
                    {
                        "type": "text",
                        # "text": "画条狗 输出两张图片",
                        # "text": "南京天气如何",
                        # "text": "https://sfile.chatglm.cn/chatglm4/3db10f79-a952-4987-83d2-cf0cfd5d5530.pdf 总结这个文件",

                        "text": "https://sfile.chatglm.cn/chatglm4/3db10f79-a952-4987-83d2-cf0cfd5d5530.pdf 基于这个文件做个ppt"

                        # "text": "周杰伦 【最新动态、相关信息或新闻】",
                        # "text": "生成PPT"
                    }]
            }
        ],

        # messages=[
        #     {
        #         "role": "user",
        #         "content": [
        #             {
        #                 "type": "text",
        #                 "text": "基于这个内容做个ppt"
        #             },
        #             {
        #                 "type": "file",
        #                 "file": [
        #                     {
        #                         "file_id": "chatglm4/3db10f79-a952-4987-83d2-cf0cfd5d5530.pdf",
        #                         "file_url": "https://sfile.chatglm.cn/chatglm4/3db10f79-a952-4987-83d2-cf0cfd5d5530.pdf",
        #                         "file_name": "附件.大模型在合规管理工作中的应用.pdf",
        #                         "file_size": 2571523,
        #                         "order": 0,
        #                         "maxReadPercent": 0,
        #                         "cover_images": [],
        #                         "url": "https://sfile.chatglm.cn/chatglm4/3db10f79-a952-4987-83d2-cf0cfd5d5530.pdf"
        #                     }
        #                 ]
        #             }
        #         ]
        #     }
        # ]
    )

    arun(Completions().create(request))
    # 输入 输出
    # input
    # {
    #     "assistant_id": "65d2f07bb2c10188f885bd89",
    #     "conversation_id": "67d932d5e579c3ded42aa80e",
    #     "meta_data": {
    #         "if_plus_model": false,
    #         "is_test": false,
    #         "input_question_type": "xxxx",
    #         "channel": "",
    #         "draft_id": "",
    #         "quote_log_id": "",
    #         "platform": "pc"
    #     },
    #     "messages": [
    #         {
    #             "role": "user",
    #             "content": [
    #                 {
    #                     "type": "text",
    #                     "text": "基于这个内容做个ppt"
    #                 },
    #                 {
    #                     "type": "file",
    #                     "file": [
    #                         {
    #                             "file_id": "chatglm4/3db10f79-a952-4987-83d2-cf0cfd5d5530.pdf",
    #                             "file_url": "https://sfile.chatglm.cn/chatglm4/3db10f79-a952-4987-83d2-cf0cfd5d5530.pdf",
    #                             "file_name": "附件.大模型在合规管理工作中的应用.pdf",
    #                             "file_size": 2571523,
    #                             "order": 0,
    #                             "maxReadPercent": 0,
    #                             "cover_images": [],
    #                             "url": "https://sfile.chatglm.cn/chatglm4/3db10f79-a952-4987-83d2-cf0cfd5d5530.pdf"
    #                         }
    #                     ]
    #                 }
    #             ]
    #         }
    #     ]
    # }



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : MeUtils.
# @File         : shake_demo
# @Time         : 2021/3/30 4:43 下午
# @Author       : yuanjie
# @WeChat       : 313303303
# @Software     : PyCharm
# @Description  : 在10万召回1万场景下，分10个片区 shake范围在 6.8%【10%以内】


import faiss
from meutils.pipe import *
from meutils.annzoo.faiss import ANN
from meutils.np_utils import normalize

np.random.seed(666)

data = normalize(np.random.random((100000, 128)).astype('float32'))
target = normalize(np.random.random((1, 128)).astype('float32'))

ann = ANN()
ann.train(data, metric=faiss.METRIC_INNER_PRODUCT, index_factory='IVF1,Flat')

N = 10000
dis, idx = ann.search(target, topK=N)
idx_list = []
for i in range(10):
    data_ = data[i * N:(i + 1) * N]
    ann_ = ANN()
    ann_.train(data_, metric=faiss.METRIC_INNER_PRODUCT, index_factory='IVF1,Flat')
    idx_ = ann_.search(target, topK=N // 10)[1] + i * N
    idx_list.append(idx_)

r = set(np.row_stack(idx_list).reshape(1, -1).tolist()[0])
len(set(idx[0].tolist()) - r)



# -*- coding: utf-8 -*-

from __future__ import absolute_import

from .base import Type
from .isobmff import IsoBmff


class Mp4(IsoBmff):
    """
    Implements the MP4 video type matcher.
    """
    MIME = 'video/mp4'
    EXTENSION = 'mp4'

    def __init__(self):
        super(Mp4, self).__init__(
            mime=Mp4.MIME,
            extension=Mp4.EXTENSION
        )

    def match(self, buf):
        if not self._is_isobmff(buf):
            return False

        major_brand, minor_version, compatible_brands = self._get_ftyp(buf)
        for brand in compatible_brands:
            if brand in ['mp41', 'mp42', 'isom']:
                return True
        return major_brand in ['mp41', 'mp42', 'isom']


class M4v(Type):
    """
    Implements the M4V video type matcher.
    """
    MIME = 'video/x-m4v'
    EXTENSION = 'm4v'

    def __init__(self):
        super(M4v, self).__init__(
            mime=M4v.MIME,
            extension=M4v.EXTENSION
        )

    def match(self, buf):
        return (len(buf) > 10 and
                buf[0] == 0x0 and buf[1] == 0x0 and
                buf[2] == 0x0 and buf[3] == 0x1C and
                buf[4] == 0x66 and buf[5] == 0x74 and
                buf[6] == 0x79 and buf[7] == 0x70 and
                buf[8] == 0x4D and buf[9] == 0x34 and
                buf[10] == 0x56)


class Mkv(Type):
    """
    Implements the MKV video type matcher.
    """
    MIME = 'video/x-matroska'
    EXTENSION = 'mkv'

    def __init__(self):
        super(Mkv, self).__init__(
            mime=Mkv.MIME,
            extension=Mkv.EXTENSION
        )

    def match(self, buf):
        contains_ebml_element = buf.startswith(b'\x1A\x45\xDF\xA3')
        contains_doctype_element = buf.find(b'\x42\x82\x88matroska') > -1
        return contains_ebml_element and contains_doctype_element


class Webm(Type):
    """
    Implements the WebM video type matcher.
    """
    MIME = 'video/webm'
    EXTENSION = 'webm'

    def __init__(self):
        super(Webm, self).__init__(
            mime=Webm.MIME,
            extension=Webm.EXTENSION
        )

    def match(self, buf):
        contains_ebml_element = buf.startswith(b'\x1A\x45\xDF\xA3')
        contains_doctype_element = buf.find(b'\x42\x82\x84webm') > -1
        return contains_ebml_element and contains_doctype_element


class Mov(IsoBmff):
    """
    Implements the MOV video type matcher.
    """
    MIME = 'video/quicktime'
    EXTENSION = 'mov'

    def __init__(self):
        super(Mov, self).__init__(
            mime=Mov.MIME,
            extension=Mov.EXTENSION
        )

    def match(self, buf):
        if not self._is_isobmff(buf):
            return False

        major_brand, minor_version, compatible_brands = self._get_ftyp(buf)
        return major_brand == 'qt  '


class Avi(Type):
    """
    Implements the AVI video type matcher.
    """
    MIME = 'video/x-msvideo'
    EXTENSION = 'avi'

    def __init__(self):
        super(Avi, self).__init__(
            mime=Avi.MIME,
            extension=Avi.EXTENSION
        )

    def match(self, buf):
        return (len(buf) > 11 and
                buf[0] == 0x52 and
                buf[1] == 0x49 and
                buf[2] == 0x46 and
                buf[3] == 0x46 and
                buf[8] == 0x41 and
                buf[9] == 0x56 and
                buf[10] == 0x49 and
                buf[11] == 0x20)


class Wmv(Type):
    """
    Implements the WMV video type matcher.
    """
    MIME = 'video/x-ms-wmv'
    EXTENSION = 'wmv'

    def __init__(self):
        super(Wmv, self).__init__(
            mime=Wmv.MIME,
            extension=Wmv.EXTENSION
        )

    def match(self, buf):
        return (len(buf) > 9 and
                buf[0] == 0x30 and
                buf[1] == 0x26 and
                buf[2] == 0xB2 and
                buf[3] == 0x75 and
                buf[4] == 0x8E and
                buf[5] == 0x66 and
                buf[6] == 0xCF and
                buf[7] == 0x11 and
                buf[8] == 0xA6 and
                buf[9] == 0xD9)


class Flv(Type):
    """
    Implements the FLV video type matcher.
    """
    MIME = 'video/x-flv'
    EXTENSION = 'flv'

    def __init__(self):
        super(Flv, self).__init__(
            mime=Flv.MIME,
            extension=Flv.EXTENSION
        )

    def match(self, buf):
        return (len(buf) > 3 and
                buf[0] == 0x46 and
                buf[1] == 0x4C and
                buf[2] == 0x56 and
                buf[3] == 0x01)


class Mpeg(Type):
    """
    Implements the MPEG video type matcher.
    """
    MIME = 'video/mpeg'
    EXTENSION = 'mpg'

    def __init__(self):
        super(Mpeg, self).__init__(
            mime=Mpeg.MIME,
            extension=Mpeg.EXTENSION
        )

    def match(self, buf):
        return (len(buf) > 3 and
                buf[0] == 0x0 and
                buf[1] == 0x0 and
                buf[2] == 0x1 and
                buf[3] >= 0xb0 and
                buf[3] <= 0xbf)


class M3gp(Type):
    """Implements the 3gp image type matcher."""

    MIME = 'video/3gpp'
    EXTENSION = '3gp'

    def __init__(self):
        super(M3gp, self).__init__(
            mime=M3gp.MIME,
            extension=M3gp.EXTENSION
        )

    def match(self, buf):
        return buf[:7] == bytearray([0x66, 0x74, 0x79, 0x70, 0x33, 0x67, 0x70])



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : videos
# @Time         : 2025/10/13 13:46
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  : 


from meutils.pipe import *
from meutils.db.redis_db import redis_aclient
from meutils.llm.clients import AsyncClient
from meutils.io.files_utils import to_url, to_url_fal
from meutils.schemas.video_types import SoraVideoRequest, Video

from openai import APIStatusError

MODELS_MAPPING = {

    "minimax-hailuo-02": "minimax:3@1",
    "minimax-hailuo-2.3": "minimax:4@1",
    "minimax-hailuo-2.3-fast": "minimax:4@2",

    "kling-2.5": "klingai:6@0",  # Kling 2.5 Turbo Standard model only supports image-to-video generation
    "kling-2.5-pro": "klingai:6@1",

    "viduq2-pro": "vidu:3@1",
    "viduq2-turbo": "vidu:3@2",

    "sora-2": "openai:3@1",
    "sora-2-pro": "openai:3@2",

    "veo3.1": "google:3@2",
    "veo3.1-fast": "google:3@3",

}

# https://runware.ai/docs/en/video-inference/api-reference#request-providersettings
provider_settings_mapping = {
    'vidu': {
        "providerSettings": {
            "vidu": {
                "movementAmplitude": "auto",
                "bgm": True,
                "style": "anime"
            }
        }
    },

    'minimax': {"providerSettings": {
        "minimax": {
            "promptOptimizer": True
        }
    }}

}


async def create_task(request: SoraVideoRequest, api_key: str, base_url: Optional[str] = None):
    # 模型路由
    if request.model.startswith("minimax"):
        mapping = {
            "512p": "512x512",
            "768p": "1366x768",
            "1080p": "1920x1080",
        }
        request.size = mapping.get(request.resolution) or request.size or "1366x768"

    elif request.model.startswith("vidu"):  # todo 视频模型 分辨率 映射
        mapping = {
            "512p": "512x512",
            "768p": "1366x768",
            "1080p": "1920x1080",
        }
        request.size = mapping.get(request.resolution) or request.size or "512x512"

    payload = [
        {
            "taskType": "videoInference",

            "taskUUID": str(uuid.uuid4()),

            "model": MODELS_MAPPING.get(request.model, request.model),
            "positivePrompt": request.prompt,

            "numberResults": 1,

            "duration": request.seconds and int(request.seconds) or 5,

            "includeCost": True,

            "webhookURL": "https://openai-dev.chatfire.cn/sys/webhook/runware?expr=%24..taskUUID",  # task_id
        },
    ]

    if provider_settings := provider_settings_mapping.get(request.model):
        payload[0]['providerSettings'] = provider_settings

    if request.model not in {"kling-2.5"}:
        if request.size:
            payload[0]["width"], payload[0]["height"] = map(int, request.size.split("x"))

    if urls := request.input_reference:  # todo 多图

        if request.model.startswith(("vidu",)):
            payload[0]["referenceImages"] = urls
        else:
            payload[0]["frameImages"] = [
                {
                    "inputImage": url
                }
                for url in urls
            ]

    logger.debug(bjson(payload))

    try:
        client = AsyncClient(base_url="https://api.runware.ai/v1", api_key=api_key, timeout=300)
        response = await client.post(
            "/",
            body=payload,
            cast_to=object
        )

        video = Video(
            id=response["data"][0]["taskUUID"],
            model=request.model,
            seconds=request.seconds,
        )

        await redis_aclient.set(video.id, api_key, ex=3600)  # 回调接口即可

        return video

    except APIStatusError as e:
        if (errors := e.response.json().get("errors")):
            logger.debug(bjson(errors))

        raise e


async def get_task(task_id):
    if not await redis_aclient.get(task_id):
        raise ValueError(f"task_id not found")

    video = Video(id=task_id)
    webhook_id = f"webhook:runware:{task_id}"
    if (data := await redis_aclient.lrange(webhook_id, 0, -1)) and (runware_response := json.loads(data[0])):
        if errors := runware_response.get("errors"):
            logger.debug(bjson(errors))
            video.status = "failed"
            video.error = errors
            return video

        if data := runware_response.get("data"):
            if video_url := data[0].get("videoURL"):
                video.status = "completed"
                video.progress = 100
                video.video_url = video_url

    return video


if __name__ == '__main__':
    model = "openai:3@1"
    # model = "viduq2-turbo"
    model = "bytedance:2@2"
    # model = "kling-2.5"

    request = SoraVideoRequest(model=model, prompt="裸体女孩", seconds="5",

                               size="864x480",

                               )

    logger.info(request)

    # arun(create_task(request, api_key="TdgJuoP9VEyeBBtzuF5q6E1ibdioHAyV"))

    # https://openai-dev.chatfire.cn/sys/webhook/runware?expr=%24..taskUUID

    task_id = "6ef0dd4b-d43e-4be8-b92c-94446ca7dcb2"
    arun(get_task(task_id))

    # {
    #     "data": [
    #         {
    #             "taskType": "videoInference",
    #             "taskUUID": "8a5a1c09-d0a5-4b1b-9b67-8943cacc935f"
    #         }
    #     ]
    # }

"""
curl --request POST \
--url 'https://api.runware.ai/v1' \
--header "Authorization: Bearer Fk3Clsgcwc3faIvbsjDajGFATJLfaWpE" \
--header "Content-Type: application/json" \
--data-raw '[
  {
    "taskType": "videoInference",
    "duration": 4,
    "fps": 30,
    "height": 720,
    "width": 1280,
    "model": "openai:3@1",
    "numberResults": 1,
    "positivePrompt": "a cat",
    "taskUUID": "3f7fcfc4-e257-4cb7-bf97-439bceaa7cc3",
    "webhookURL": "https://openai-dev.chatfire.cn/sys/webhook/runware?expr=%24..taskUUID"
  }
]'


{
  "id": "12ad6eb9-cd3b-47d8-887b-b9d84f2e291d",
  "error": [
    {
      "code": "providerError",
      "message": "OpenAI responded with an error. The provider returned the video_generation_failed. Additional information below.",
      "taskUUID": "12ad6eb9-cd3b-47d8-887b-b9d84f2e291d",
      "documentation": "https://runware.ai/docs/en/providers/introduction",
      "responseContent": "Video generation failed due to an internal error.",
      "responseStatusCode": "video_generation_failed"
    }
  ],
  "object": "video",
  "status": "failed",
  "created_at": 1762940611
}
"""



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : demo
# @Time         : 2023/12/12 10:14
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  : 

from meutils.pipe import *
from sentence_transformers import SentenceTransformer

sentences = ["我是中国人", "中国人是我"]
model = SentenceTransformer('infgrad/stella-large-zh-v2')
print(model.max_seq_length)
embeddings_1 = model.encode(sentences, normalize_embeddings=True)
embeddings_2 = model.encode(sentences, normalize_embeddings=True)
similarity = embeddings_1 @ embeddings_2.T
print(similarity)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : yezi
# @Time         : 2024/6/28 09:23
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# todo: 接码平台
# @Description  : https://github.com/Thekers/Get_OpenaiKey/blob/9d174669d7778ea32d1132bedd5167597912dcfb/Add_01AI_Token.py
# https://note.youdao.com/ynoteshare/index.html?id=543e0ee9d51faf3d4c33d792971d9de9&type=note&_time=1719535893843
import os

import httpx

from meutils.pipe import *

"http://api.sqhyw.net:90/api/get_myinfo?token=xxxxxxx"

BASE_URL = "http://api.sqhyw.net:90"


class Request(BaseModel):
    token: str
    project_id: str
    loop: str = 1
    operator: str = 4  # 0=默认 1=移动 2=联通 3=电信 4=实卡 5=虚卡
    phone_num: Optional[str] = None  # 指定取号的话 这里填要取的手机号
    scope: Optional[str] = None  # 指定号段 最多支持号码前5位. 例如可以为165，也可以为16511address
    address: Optional[str] = None  # 归属地选择 例如 湖北 甘肃 不需要带省字
    scope_black: Optional[str] = None  # 排除号段最长支持7位且可以支持多个,最多支持20个号段。用逗号分隔 比如150,1511111,15522
    api_id: Optional[str] = None  # 如果是开发者,此处填写你的用户ID才有收益，注意是用户ID不是登录账号


@alru_cache()
async def get_access_token():
    username, password = os.getenv("YEZI_USER").split('|')
    payload = {
        "username": username,
        "password": password
    }
    async with httpx.AsyncClient(base_url=BASE_URL) as client:
        response = await client.get("/api/logins", params=payload)
        if response.is_success:
            logger.debug(response.json())
            return response.json().get("token")


async def get_mobile(project_id: str = "806053"):
    token = await get_access_token()
    payload = Request(token=token, project_id=project_id).model_dump()

    async with httpx.AsyncClient(base_url=BASE_URL) as client:
        response = await client.get("/api/get_mobile", params=payload)
        if response.is_success:
            logger.debug(response.json())
            return response.json().get("mobile")


async def get_message(project_id, phone_num):
    token = await get_access_token()
    payload = Request(token=token, project_id=project_id, phone_num=phone_num).model_dump()

    async with httpx.AsyncClient(base_url=BASE_URL) as client:
        s = time.time()
        while 1:
            await asyncio.sleep(1)
            response = await client.get("/api/get_message", params=payload)
            if response.is_success and (code := response.json().get("code")):
                logger.debug(response.json())
                return code
            if time.time() - s > 120:
                logger.error("获取短信验证码失败")
                break


async def free_mobile(project_id, phone_num: Optional[str] = None):  # 释放所有
    token = await get_access_token()
    payload = Request(token=token, project_id=project_id, phone_num=phone_num).model_dump()

    async with httpx.AsyncClient(base_url=BASE_URL) as client:
        response = await client.get("/api/free_mobile", params=payload)
        if response.is_success:
            logger.debug("释放")


async def get_code(project_id):
    phone_num = await get_mobile(project_id)
    code = await get_message(project_id, phone_num)
    await free_mobile(project_id, phone_num)
    return code


if __name__ == '__main__':
    # print(arun(get_access_token()))
    # print(arun(get_mobile()))
    print(arun(get_message(project_id="806053", phone_num="19211569315")))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : chat
# @Time         : 2025/11/24 11:35
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  : 

from meutils.pipe import *
from meutils.schemas.openai_types import CompletionRequest, CompletionUsage
import replicate


class Completions(object):

    def __init__(self, api_key: Optional[str] = None, base_url: Optional[str] = None):
        api_key = api_key or os.getenv("REPLICATE_API_KEY")
        self.client = replicate.client.Client(api_token=api_key)

    async def create(self, request: CompletionRequest):

        payload = {
            # "audio": "https://replicate.delivery/pbxt/O5Vw2eTOp7z4V27QYXqEUQZ5OvwTEKj2TVf3syi4dTJpvUG9/Never%20Gonna%20Give%20You%20Up%20-%20Rick%20Astley.mp3",
            "top_p": request.top_p,
            "images": [],
            "prompt": request.last_user_content,
            "videos": [],
            "temperature": request.temperature,
            "thinking_level": request.reasoning_effort or "low",
            "max_output_tokens": request.max_completion_tokens or 64000
        }

        if image_urls := request.last_urls.get("image_url"):
            payload["images"] += image_urls
        elif audio_urls := request.last_urls.get("audio_url"):
            payload["audio"] = audio_urls[0]
        elif video_urls := request.last_urls.get("video_url"):
            payload["videos"].append(video_urls[0])  # 默认第一个

        logger.debug(bjson(payload))

        if request.stream:
            chunks = await self.client.async_stream(request.model, input=payload, wait=False)
            async for chunk in chunks:
                yield str(chunk)
        else:
            chunks = await self.client.async_run(request.model, input=payload, wait=False)
            _ = ''.join(chunks)
            logger.debug(_)
            yield _


if __name__ == '__main__':
    url = "https://lmdbk.com/5.mp4"
    request = CompletionRequest(
        model="google/gemini-3-pro",
        stream=True,
        messages=[{"role": "user", "content": "你好"}])

    arun(Completions().create(request))



# -*- coding: utf-8 -*-

from __future__ import absolute_import

from .types import ARCHIVE as archive_matchers
from .types import AUDIO as audio_matchers
from .types import APPLICATION as application_matchers
from .types import DOCUMENT as document_matchers
from .types import FONT as font_matchers
from .types import IMAGE as image_matchers
from .types import VIDEO as video_matchers
from .types import TYPES
from .utils import get_bytes


def match(obj, matchers=TYPES):
    """
    Matches the given input against the available
    file type matchers.

    Args:
        obj: path to file, bytes or bytearray.

    Returns:
        Type instance if type matches. Otherwise None.

    Raises:
        TypeError: if obj is not a supported type.
    """
    buf = get_bytes(obj)

    for matcher in matchers:
        if matcher.match(buf):
            return matcher

    return None


def image_match(obj):
    """
    Matches the given input against the available
    image type matchers.

    Args:
        obj: path to file, bytes or bytearray.

    Returns:
        Type instance if matches. Otherwise None.

    Raises:
        TypeError: if obj is not a supported type.
    """
    return match(obj, image_matchers)


def font_match(obj):
    """
    Matches the given input against the available
    font type matchers.

    Args:
        obj: path to file, bytes or bytearray.

    Returns:
        Type instance if matches. Otherwise None.

    Raises:
        TypeError: if obj is not a supported type.
    """
    return match(obj, font_matchers)


def video_match(obj):
    """
    Matches the given input against the available
    video type matchers.

    Args:
        obj: path to file, bytes or bytearray.

    Returns:
        Type instance if matches. Otherwise None.

    Raises:
        TypeError: if obj is not a supported type.
    """
    return match(obj, video_matchers)


def audio_match(obj):
    """
    Matches the given input against the available
    autio type matchers.

    Args:
        obj: path to file, bytes or bytearray.

    Returns:
        Type instance if matches. Otherwise None.

    Raises:
        TypeError: if obj is not a supported type.
    """
    return match(obj, audio_matchers)


def archive_match(obj):
    """
    Matches the given input against the available
    archive type matchers.

    Args:
        obj: path to file, bytes or bytearray.

    Returns:
        Type instance if matches. Otherwise None.

    Raises:
        TypeError: if obj is not a supported type.
    """
    return match(obj, archive_matchers)


def application_match(obj):
    """
    Matches the given input against the available
    application type matchers.

    Args:
        obj: path to file, bytes or bytearray.

    Returns:
        Type instance if matches. Otherwise None.

    Raises:
        TypeError: if obj is not a supported type.
    """
    return match(obj, application_matchers)


def document_match(obj):
    """
    Matches the given input against the available
    document type matchers.

    Args:
        obj: path to file, bytes or bytearray.

    Returns:
        Type instance if matches. Otherwise None.

    Raises:
        TypeError: if obj is not a supported type.
    """
    return match(obj, document_matchers)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : utils
# @Time         : 2025/10/31 17:57
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  : 

from meutils.pipe import *
from meutils.str_utils.json_utils import json_path


def to_status(result, default_status="queued", mode: str = "openai") -> str:
    """

    :param result: response
    :param default_status:
    :param mode:
        openai "queued", "in_progress", "completed", "failed" => 映射到其他
    :return:
    """
    if isinstance(result, str):
        result = {"status": result}

    if status := (
            result.get("status")
            or result.get("task_status")
            or result.get("state")
            or result.get("task_state")
            or json_path(result, expr="$..status")

    ):
        if isinstance(status, list):
            status = status[0]

        if isinstance(status, dict):  # fal
            status = "failed"

        status = str(status).lower()
        logger.debug(status)

        if status.startswith(("pro", "inpro", "pending", "task_status_queu", "sub", "start", "run", "inqueue", "queu")):
            status = "queued"

        if status.startswith(("succ", "ok", "compl", "task_status_succ")):
            status = "completed"

        if status.startswith(("fail", "error", "cancel", "task_status_fail")):
            status = "failed"

        if any(i in status for i in ("moder",)):  # 内容审核
            status = "failed"

        if any(i in status for i in ("feature_not_supported",)):
            status = "failed"

    return status or default_status


if __name__ == '__main__':
    result = {'id': 'cgt-20250613160030-2dvd7',
 'model': 'doubao-seedance-1-0-pro-250528',
 'status': 'succeeded',
 'content': {'video_url': 'https://ark-content-generation-cn-beijing.tos-cn-beijing.volces.com/doubao-seedance-1-0-pro/02174980163157800000000000000000000ffffac182c17b26890.mp4?X-Tos-Algorithm=TOS4-HMAC-SHA256&X-Tos-Credential=AKLTYjg3ZjNlOGM0YzQyNGE1MmI2MDFiOTM3Y2IwMTY3OTE%2F20250613%2Fcn-beijing%2Ftos%2Frequest&X-Tos-Date=20250613T080120Z&X-Tos-Expires=86400&X-Tos-Signature=5e0928f738f49b93f54923549de4c65940c5007d5e86cb5ebadc756cca3aa03e&X-Tos-SignedHeaders=host'},
 'usage': {'completion_tokens': 246840, 'total_tokens': 246840},
 'created_at': 1749801631,
 'updated_at': 1749801680}
    _ = to_status(result)

    logger.debug(_)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : MeUtils.
# @File         : machine_monitor
# @Time         : 2022/7/14 上午9:03
# @Author       : yuanjie
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  : https://blog.csdn.net/Deaohst/article/details/119132135


import numpy as np
import psutil
from functools import wraps


def source_info():
    cpu_per = psutil.cpu_percent(True, True)
    cpu_per = round(np.array(cpu_per).mean(), 2)
    mem = psutil.virtual_memory()
    cpu_num = psutil.cpu_count(logical=True)
    mem_total = round(mem.total / 2 ** 32, 2)
    mem_per = round(mem.percent, 2)
    disk = []
    partitions = psutil.disk_partitions()
    for i in partitions:
        info = psutil.disk_usage(i[1])
        disk.append([info.total, info.used])
    disk = np.array(disk)
    disk = disk.sum(axis=0)
    disk_pre = round(disk[1] / disk[0] * 100, 2)
    disk_total = round(disk[0] / 2 ** 30, 2)

    return {
        "cpu数量": f"{cpu_num} | {cpu_per}%",
        "内存总量": f"{mem_total}GB | {mem_per}%",
        "磁盘总量": f"{disk_total}GB | {disk_pre}%"
    }


def dev_source_monitor(f):
    @wraps(f)
    def decorated(*args, **kwargs):
        cpu_per = psutil.cpu_percent(True, True)
        cpu_per = round(np.array(cpu_per).mean(), 2)
        mem = psutil.virtual_memory()
        cpu_num = psutil.cpu_count(logical=True)
        mem_total = round(mem.total / 2 ** 32, 2)
        mem_per = round(mem.percent, 2)
        disk = []
        partitions = psutil.disk_partitions()
        for i in partitions:
            info = psutil.disk_usage(i[1])
            disk.append([info.total, info.used])
        disk = np.array(disk)
        disk = disk.sum(axis=0)
        disk_pre = round(disk[1] / disk[0] * 100, 2)
        disk_total = round(disk[0] / 2 ** 30, 2)
        print('设备资源总量')
        print(f"cpu数量:{cpu_num}")
        print(f"内存总量:{mem_total}GB")
        print(f"磁盘总量:{disk_total}GB")
        print("设备资源使用")
        print(f"cpu使用:{cpu_per}%")
        print(f"内存使用:{mem_per}%")
        print(f"磁盘使用:{disk_pre}%")
        # print(decorated.__name__)
        return f(*args, **kwargs)

    return decorated


if __name__ == '__main__':
    # @dev_source_monitor
    # def func():
    #     pass
    #
    #
    # func()

    print(source_info())



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : check
# @Time         : 2025/11/12 14:02
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  : 

from meutils.pipe import *
from openai import Client

payload = [
  {
    "taskType": "accountManagement",
    "taskUUID": str(uuid.uuid4()),
    "operation": "getDetails"
  }
]


api_key="P8nkXmMKfoPlSR3iR48Jbl4vs8aLP1TT"

client = Client(base_url="https://api.runware.ai/v1", api_key=api_key, timeout=300)
response = client.post(
    "/",
    body=payload,
    cast_to=object
)

response['data'][0]['balance']


#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : minimax
# @Time         : 2025/5/27 18:37
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  :
"""
T2A v2（语音生成）

T2A Large v2（异步超长文本语音生成）

快速复刻（Voice Cloning）

Voice Generation（文生音色）

"""
import os

from meutils.pipe import *
from meutils.decorators.retry import retrying

from meutils.io.files_utils import to_bytes, to_url
from meutils.schemas.openai_types import TTSRequest

BASE_URL = os.getenv("MINIMAX_BASE_URL") or "https://api.minimax.chat/v1"


@retrying()
async def create_tts(payload: dict, token: Optional[str] = None):
    """https://platform.minimaxi.com/document/T2A%20V2?key=66719005a427f0c8a5701643"""
    group_id, api_key = (token or os.getenv("MINIMAX_API_KEY")).split('|')

    headers = {
        "Authorization": f"Bearer {api_key}",
        "Content-Type": "application/json"
    }

    async with httpx.AsyncClient(base_url=BASE_URL, headers=headers, timeout=300) as client:
        response = await client.post(f"/t2a_v2?GroupId={group_id}", json=payload)
        response.raise_for_status()
        """
        状态码。1000，未知错误；1001，超时；1002，触发限流；1004，鉴权失败；1039，触发TPM限流；1042，非法字符超过10%；2013，输入格式信息不正常。
        """

        data = response.json()
        return data


async def create_tts_for_openai(request: TTSRequest, token: Optional[str] = None):
    if request.instructions:
        if len(request.input) < 500:
            data = await text2voice(request, token)
            _ = bytes.fromhex(data["trial_audio"])
            if request.response_format == "url":
                return {"data": {"audio": await to_url(_, filename=f'{shortuuid.random()}.mp3')}}
            else:
                return _
        else:
            request_input = request.input
            request.input = request_input[:500]
            data = await text2voice(request, token)

            logger.debug("text2voice => tts")
            request.instructions = None  # 跳出递归
            request.voice = data["voice_id"]
            request.input = request_input
            return await create_tts_for_openai(request, token)

    payload = {
        # speech-02-hd、speech-02-turbo、speech-01-hd、speech-01-turbo、speech-01-240228、speech-01-turbo-240228
        "model": request.model,
        "text": request.input,
        "timber_weights": [
            {
                "voice_id": request.voice or "male-qn-qingse",
                "weight": 100
            }
        ],
        "voice_setting": {
            "voice_id": "",
            "speed": request.speed,  # 范围[0.5,2]，默认值为1.0
            "pitch": 0,
            "vol": 1,
            "latex_read": False,

            # 当前支持7种情绪：高兴，悲伤，愤怒，害怕，厌恶，惊讶，中性；
            # 参数范围["happy", "sad", "angry", "fearful", "disgusted", "surprised", "neutral"]
            # "emotion": "neutral",
        },
        "audio_setting": {
            "sample_rate": 32000,
            "bitrate": 128000,
            "format": "mp3"
        },
        "language_boost": "auto"
    }

    data = await create_tts(payload, token)

    if request.response_format == "url":
        if data.get("base_resp").get("status_code") == 0:
            _ = bytes.fromhex(data["data"]["audio"])
            data["data"]["audio"] = await to_url(_, filename=f'{shortuuid.random()}.mp3')
        return data
    else:
        data = bytes.fromhex(data["data"]["audio"])
        return data


async def text2voice(request: TTSRequest, token: Optional[str] = None):
    group_id, api_key = (token or os.getenv("MINIMAX_API_KEY")).split('|')

    if "female" in request.voice:
        request.voice = "female"
    elif "male" in request.voice:
        request.voice = "male"
    else:
        request.voice = "female"

    # child

    headers = {
        "Authorization": f"Bearer {api_key}",
        "Content-Type": "application/json"
    }
    payload = {
        "gender": request.voice,

        # 1.child、2.teenager、3.young、4.middle-aged、5.old。
        "age": "young",

        "voice_desc": [request.instructions],

        "text": request.input
    }

    async with httpx.AsyncClient(base_url=BASE_URL, headers=headers, timeout=300) as client:
        response = await client.post("/text2voice", json=payload)
        response.raise_for_status()
        """
        {
            "trial_audio": "hex编码音频",
            "voice_id": "voiceID",
            "base_resp": {
                "status_code": 0,
                "status_msg": "success"
            }
        }        
        """

        data = response.json()

        # logger.debug(await to_url(bytes.fromhex(data["trial_audio"]), filename=f'{shortuuid.random()}.mp3'))

        return data


if __name__ == '__main__':
    payload = {
        "model": "speech-02-hd",
        "text": "人工智能不是要替代人类，而是要增强人类的能力。",
        "timber_weights": [
            {
                "voice_id": "Boyan_new_platform",
                "weight": 100
            }
        ],
        "voice_setting": {
            "voice_id": "",
            "speed": 1,
            "pitch": 0,
            "vol": 1,
            "latex_read": False
        },
        "audio_setting": {
            "sample_rate": 32000,
            "bitrate": 128000,
            "format": "mp3"
        },
        "language_boost": "auto"
    }

    # arun(create_tts(payload))

    request = TTSRequest(
        model="speech-02-hd",
        # input="你好呀" * 100,
        input="你好呀" * 200,

        instructions="委婉",

        voice='male',

        response_format="url"
    )

    arun(create_tts_for_openai(request))

    # arun(text2voice(request))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : async_tasks
# @Time         : 2025/7/28 16:00
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  : 

from meutils.pipe import *
from meutils.db.redis_db import redis_aclient
from meutils.io.files_utils import to_bytes

from meutils.apis.utils import make_request_httpx
from meutils.schemas.gitee_types import FEISHU_URL, BASE_URL
from meutils.schemas.image_types import ImageRequest, ImagesResponse

from meutils.config_utils.lark_utils import get_next_token_for_polling

"""
curl https://ai.gitee.com/v1/async/image-to-3d \
	-X POST \
	-H "Authorization: Bearer XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX" \
	-F "image=@path/to/image.jpg" \
	-F "type=glb" \
	-F "model=Hi3DGen" \
	-F "seed=1234" \
	-F "file_format=glb"
	
	
texture=true 带纹理
"""


async def get_task(task_id):
    if api_key := await redis_aclient.get(task_id):
        api_key = api_key.decode()

        headers = {
            "Authorization": f"Bearer {api_key}",
        }
        response = await make_request_httpx(
            base_url=BASE_URL,
            path=f"/task/{task_id}",
            headers=headers,
        )
        """
        {'completed_at': 1753692159067,
 'created_at': 1753692151094,
 'output': {'file_url': 'https://gitee-ai.su.bcebos.com/serverless-api/2025-07-28/GB84DX8LK6NUJ0WHZLUNRCXDBFKMVVFH.glb?authorization=bce-auth-v1%2FALTAKZc1TWR1oEpkHMlwBs5YXU%2F2025-07-28T08%3A42%3A38Z%2F604800%2F%2F2799bb11463d736d0eb0fd656c944e02012f7359ca22e8397811f59d306ec353'},
 'started_at': 1753692151358,
 'status': 'success',
 'task_id': 'GB84DX8LK6NUJ0WHZLUNRCXDBFKMVVFH',
 'urls': {'cancel': 'https://ai.gitee.com/api/v1/task/GB84DX8LK6NUJ0WHZLUNRCXDBFKMVVFH/cancel',
          'get': 'https://ai.gitee.com/api/v1/task/GB84DX8LK6NUJ0WHZLUNRCXDBFKMVVFH'}}
        """
        response.pop('urls', None)
        return response


async def create_task(image, data: Optional[dict] = None, api_key: Optional[str] = None):
    api_key = api_key or await get_next_token_for_polling(FEISHU_URL)
    headers = {
        "Authorization": f"Bearer {api_key}",
    }

    # (filename, file_bytes, mime_type) = image

    response = await make_request_httpx(
        base_url=BASE_URL,
        path="/async/image-to-3d",
        headers=headers,
        files={
            "image": image,
        },
        data=data,
    )
    """
    {'created_at': 1753691466227,
 'status': 'waiting',
 'task_id': 'AXA5X0GFGXS96T4ALPYHCTCHAAYANL4U',
 'urls': {'cancel': 'https://ai.gitee.com/api/v1/task/AXA5X0GFGXS96T4ALPYHCTCHAAYANL4U/cancel',
          'get': 'https://ai.gitee.com/api/v1/task/AXA5X0GFGXS96T4ALPYHCTCHAAYANL4U'}}

{'task_id': 'GB84DX8LK6NUJ0WHZLUNRCXDBFKMVVFH'}

    """
    logger.debug(response)
    if task_id := response.get("task_id"):
        await redis_aclient.set(task_id, api_key, ex=24 * 3600)
        return {"task_id": response.get("task_id")}


async def generate(request: ImageRequest, api_key: Optional[str] = None):
    payload = request.model_dump(exclude_none=True, exclude={"extra_fields", "controls"})

    payload = {
        "type": "glb",
        "model": request.model,
        "file_format": request.response_format if request.response_format in ["glb", "stl"] else "glb",

        **payload,
        **(request.extra_fields or {})
    }

    image = await to_bytes(payload.pop('image'))

    logger.debug(payload)
    response = await create_task(image=image, data=payload)
    if response:
        for i in range(100):
            await asyncio.sleep(3)
            _ = await get_task(response['task_id'])
            logger.debug(bjson(_))
            if file_url := (_.get("output") or {}).get("file_url"):
                return ImagesResponse(data=[{"url": file_url}])


if __name__ == '__main__':
    image = "/Users/betterme/PycharmProjects/AI/test.png"
    # arun(get_task('GB84DX8LK6NUJ0WHZLUNRCXDBFKMVVFH'))
    # image = ('x.png', open(image, 'rb').read(), 'image/png')
    image = open(image, 'rb').read()
    image = "https://s3.ffire.cc/files/christmas-tree.png"

    # image = None
    arun(
        create_task(
            # image=open(image, 'rb'),
            image=image,

            data={
                "type": "glb",
                "model": "Hunyuan3D-2",
                "file_format": "glb",
            }
        ))

    # arun(generate(
    #     ImageRequest(
    #         model="Hunyuan3D-2",
    #         response_format="glb",
    #         extra_fields={
    #             "image": image,
    #
    #             "texture": True,
    #         }
    #     )
    # )
    # )





from typing import Union

from fastapi import status
from fastapi.exceptions import RequestValidationError
from fastapi.openapi.constants import REF_PREFIX
from fastapi.openapi.utils import validation_error_response_definition
from pydantic import ValidationError
from starlette.requests import Request
from starlette.responses import JSONResponse
from starlette.status import HTTP_422_UNPROCESSABLE_ENTITY


async def http422_error_handler(
        _: Request,
        exc: Union[RequestValidationError, ValidationError],
) -> JSONResponse:
    return JSONResponse(
        {"errors": exc.errors()},
        status_code=HTTP_422_UNPROCESSABLE_ENTITY,
    )


validation_error_response_definition["properties"] = {
    "errors": {
        "title": "Errors",
        "type": "array",
        "items": {"$ref": "{0}ValidationError".format(REF_PREFIX)},
    },
}


async def validation_exception_handler(_: Request, exc: RequestValidationError) -> JSONResponse:
    # print(exc)
    content = {
        "error":
            {
                "message": f"{exc}",
                "type": "RequestValidationError",
            }
    }
    return JSONResponse(
        content=content,
        status_code=status.HTTP_400_BAD_REQUEST
    )



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : MeUtils.
# @File         : wechat
# @Time         : 2021/6/7 11:17 上午
# @Author       : yuanjie
# @WeChat       : 313303303
# @Software     : PyCharm
# @Description  :

from meutils.pipe import *
from meutils.hash_utils import *


class Text(BaseConfig):
    content = ""


class Bot(object):

    def __init__(self, key='1eb25317-39a1-4af7-a6e3-63877ec2dd64'):
        self.key = key
        self.hook_url = f'https://qyapi.weixin.qq.com/cgi-bin/webhook/send?key={key}'

    def send(self, body=None):
        if body is None:
            body = {
                "msgtype": "text",
                "text": {
                    "content": "南京今日天气：29度，大部分多云，降雨概率：60%",
                    "mentioned_list": ["yuanjie", "@all"],
                    "mentioned_mobile_list": ["18550288233", "@all"]
                }
            }
        return requests.post(url=self.hook_url, json=body)

    def send_news(self):
        # json = {
        #     "msgtype": "news",
        #     "news": {
        #         "articles": [
        #             {
        #                 "title": "中秋节礼品领取",
        #                 "description": "今年中秋节公司有豪礼相送",
        #                 "url": "www.qq.com",
        #                 "picurl": "http://res.mail.qq.com/node/ww/wwopenmng/images/independent/doc/test_pic_msg1.png"
        #             }
        #         ]
        #     }
        # }
        pass

    def send_markdown(self):
        json = {
            "msgtype": "markdown",
            "markdown": {
                "content": """
                 [点击参加周会](https://meeting.tencent.com/s/UJ5c5nOdCJI3)
                 """.strip(),
                "mentioned_list": ["wangqing", "@all"],  # 不支持艾特
                "mentioned_mobile_list": ["18550288233", "@all"]
            }
        }
        return self.send(json)

    def send_image(self, path='http://www.nesc.cn/dbzq/images/logo.png'):
        _, bytes_data = self._get_bytes(path)
        body = {
            "msgtype": "image",
            "image": {
                "base64": bytes2base64(bytes_data),
                "md5": md5(bytes_data, False)
            }
        }
        return self.send(body)

    def send_file(self, path, type='file'):
        upload_media_url = f'https://qyapi.weixin.qq.com/cgi-bin/webhook/upload_media?key={self.key}&type={type}'

        name, bytes_data = self._get_bytes(path)
        files = {name: bytes_data}

        response = requests.post(upload_media_url, files=files)
        media_id = response.json()['media_id']

        body = {"msgtype": type, type: {"media_id": media_id}}
        return self.send(body)

    def _get_bytes(self, path):
        if path.startswith('http'):
            bytes_data = requests.get(path).content
        else:
            bytes_data = Path(path).read_bytes()
        return Path(path).name, bytes_data


if __name__ == '__main__':
    bot = Bot()
    bot.send()
    # bot.send_image("https://www.baidu.com/img/flexible/logo/pc/result.png")
    # bot.send_file('wechat.py')
    # bot.send_markdown()



# encoding:utf-8

"""
wechat channel
"""

import io
import json
import os
import threading
import time
import requests

from bridge.context import *
from bridge.reply import *
from channel.chat_channel import ChatChannel
from channel import chat_channel
from channel.wechat.wechat_message import *
from common.expired_dict import ExpiredDict
from common.log import logger
from common.singleton import singleton
from common.time_check import time_checker
from common.utils import convert_webp_to_png
from config import conf, get_appdata_dir
from lib import itchat
from lib.itchat.content import *


@itchat.msg_register([TEXT, VOICE, PICTURE, NOTE, ATTACHMENT, SHARING])
def handler_single_msg(msg):
    try:
        cmsg = WechatMessage(msg, False)
    except NotImplementedError as e:
        logger.debug("[WX]single message {} skipped: {}".format(msg["MsgId"], e))
        return None
    WechatChannel().handle_single(cmsg)
    return None


@itchat.msg_register([TEXT, VOICE, PICTURE, NOTE, ATTACHMENT, SHARING], isGroupChat=True)
def handler_group_msg(msg):
    ###############################################################

    from meutils.hooks.wechat import create_reply

    hook = create_reply(msg, itchat_send=itchat.send)
    if hook: return

    ###############################################################
    try:
        cmsg = WechatMessage(msg, True)
    except NotImplementedError as e:
        logger.debug("[WX]group message {} skipped: {}".format(msg["MsgId"], e))
        return None
    WechatChannel().handle_group(cmsg)
    return None


def _check(func):
    def wrapper(self, cmsg: ChatMessage):
        msgId = cmsg.msg_id
        if msgId in self.receivedMsgs:
            logger.info("Wechat message {} already received, ignore".format(msgId))
            return
        self.receivedMsgs[msgId] = True
        create_time = cmsg.create_time  # 消息时间戳
        if conf().get("hot_reload") == True and int(create_time) < int(time.time()) - 60:  # 跳过1分钟前的历史消息
            logger.debug("[WX]history message {} skipped".format(msgId))
            return
        if cmsg.my_msg and not cmsg.is_group:
            logger.debug("[WX]my message {} skipped".format(msgId))
            return
        return func(self, cmsg)

    return wrapper


# 可用的二维码生成接口
# https://api.qrserver.com/v1/create-qr-code/?size=400×400&data=https://www.abc.com
# https://api.isoyu.com/qr/?m=1&e=L&p=20&url=https://www.abc.com
def qrCallback(uuid, status, qrcode):
    # logger.debug("qrCallback: {} {}".format(uuid,status))
    if status == "0":
        try:
            from PIL import Image

            img = Image.open(io.BytesIO(qrcode))
            _thread = threading.Thread(target=img.show, args=("QRCode",))
            _thread.setDaemon(True)
            _thread.start()
        except Exception as e:
            pass

        import qrcode

        url = f"https://login.weixin.qq.com/l/{uuid}"

        qr_api1 = "https://api.isoyu.com/qr/?m=1&e=L&p=20&url={}".format(url)
        qr_api2 = "https://api.qrserver.com/v1/create-qr-code/?size=400×400&data={}".format(url)
        qr_api3 = "https://api.pwmqr.com/qrcode/create/?url={}".format(url)
        qr_api4 = "https://my.tv.sohu.com/user/a/wvideo/getQRCode.do?text={}".format(url)
        print("You can also scan QRCode in any website below:")
        print(qr_api3)
        print(qr_api4)
        print(qr_api2)
        print(qr_api1)

        _send_qr_code([qr_api3, qr_api4, qr_api2, qr_api1])

        qr = qrcode.QRCode(border=1)
        qr.add_data(url)
        qr.make(fit=True)
        qr.print_ascii(invert=True)


@singleton
class WechatChannel(ChatChannel):
    NOT_SUPPORT_REPLYTYPE = []

    def __init__(self):
        super().__init__()
        self.receivedMsgs = ExpiredDict(conf().get("expires_in_seconds", 3600))
        self.auto_login_times = 0

    def startup(self):
        try:
            itchat.instance.receivingRetryCount = 600  # 修改断线超时时间
            # login by scan QRCode
            hotReload = conf().get("hot_reload", False)
            status_path = os.path.join(get_appdata_dir(), "itchat.pkl")
            itchat.auto_login(
                enableCmdQR=2,
                hotReload=hotReload,
                statusStorageDir=status_path,
                qrCallback=qrCallback,
                exitCallback=self.exitCallback,
                loginCallback=self.loginCallback
            )
            self.user_id = itchat.instance.storageClass.userName
            self.name = itchat.instance.storageClass.nickName
            logger.info("Wechat login success, user_id: {}, nickname: {}".format(self.user_id, self.name))
            # start message listener
            itchat.run()
        except Exception as e:
            logger.exception(e)

    def exitCallback(self):
        try:
            from common.linkai_client import chat_client
            if chat_client.client_id and conf().get("use_linkai"):
                _send_logout()
                time.sleep(2)
                self.auto_login_times += 1
                if self.auto_login_times < 100:
                    chat_channel.handler_pool._shutdown = False
                    self.startup()
        except Exception as e:
            pass

    def loginCallback(self):
        logger.debug("Login success")
        _send_login_success()

    # handle_* 系列函数处理收到的消息后构造Context，然后传入produce函数中处理Context和发送回复
    # Context包含了消息的所有信息，包括以下属性
    #   type 消息类型, 包括TEXT、VOICE、IMAGE_CREATE
    #   content 消息内容，如果是TEXT类型，content就是文本内容，如果是VOICE类型，content就是语音文件名，如果是IMAGE_CREATE类型，content就是图片生成命令
    #   kwargs 附加参数字典，包含以下的key：
    #        session_id: 会话id
    #        isgroup: 是否是群聊
    #        receiver: 需要回复的对象
    #        msg: ChatMessage消息对象
    #        origin_ctype: 原始消息类型，语音转文字后，私聊时如果匹配前缀失败，会根据初始消息是否是语音来放宽触发规则
    #        desire_rtype: 希望回复类型，默认是文本回复，设置为ReplyType.VOICE是语音回复
    @time_checker
    @_check
    def handle_single(self, cmsg: ChatMessage):
        # filter system message
        if cmsg.other_user_id in ["weixin"]:
            return
        if cmsg.ctype == ContextType.VOICE:
            if conf().get("speech_recognition") != True:
                return
            logger.debug("[WX]receive voice msg: {}".format(cmsg.content))
        elif cmsg.ctype == ContextType.IMAGE:
            logger.debug("[WX]receive image msg: {}".format(cmsg.content))
        elif cmsg.ctype == ContextType.PATPAT:
            logger.debug("[WX]receive patpat msg: {}".format(cmsg.content))
        elif cmsg.ctype == ContextType.TEXT:
            logger.debug("[WX]receive text msg: {}, cmsg={}".format(json.dumps(cmsg._rawmsg, ensure_ascii=False), cmsg))
        else:
            logger.debug("[WX]receive msg: {}, cmsg={}".format(cmsg.content, cmsg))
        context = self._compose_context(cmsg.ctype, cmsg.content, isgroup=False, msg=cmsg)
        if context:
            self.produce(context)

    @time_checker
    @_check
    def handle_group(self, cmsg: ChatMessage):
        if cmsg.ctype == ContextType.VOICE:
            if conf().get("group_speech_recognition") != True:
                return
            logger.debug("[WX]receive voice for group msg: {}".format(cmsg.content))
        elif cmsg.ctype == ContextType.IMAGE:
            logger.debug("[WX]receive image for group msg: {}".format(cmsg.content))
        elif cmsg.ctype in [ContextType.JOIN_GROUP, ContextType.PATPAT, ContextType.ACCEPT_FRIEND,
                            ContextType.EXIT_GROUP]:
            logger.debug("[WX]receive note msg: {}".format(cmsg.content))
        elif cmsg.ctype == ContextType.TEXT:
            # logger.debug("[WX]receive group msg: {}, cmsg={}".format(json.dumps(cmsg._rawmsg, ensure_ascii=False), cmsg))
            pass
        elif cmsg.ctype == ContextType.FILE:
            logger.debug(f"[WX]receive attachment msg, file_name={cmsg.content}")
        else:
            logger.debug("[WX]receive group msg: {}".format(cmsg.content))
        context = self._compose_context(cmsg.ctype, cmsg.content, isgroup=True, msg=cmsg)
        if context:
            self.produce(context)

    # 统一的发送函数，每个Channel自行实现，根据reply的type字段发送不同类型的消息
    def send(self, reply: Reply, context: Context):
        receiver = context["receiver"]
        if reply.type == ReplyType.TEXT:
            itchat.send(reply.content, toUserName=receiver)
            logger.info("[WX] sendMsg={}, receiver={}".format(reply, receiver))
        elif reply.type == ReplyType.ERROR or reply.type == ReplyType.INFO:
            itchat.send(reply.content, toUserName=receiver)
            logger.info("[WX] sendMsg={}, receiver={}".format(reply, receiver))
        elif reply.type == ReplyType.VOICE:
            itchat.send_file(reply.content, toUserName=receiver)
            logger.info("[WX] sendFile={}, receiver={}".format(reply.content, receiver))
        elif reply.type == ReplyType.IMAGE_URL:  # 从网络下载图片
            img_url = reply.content
            logger.debug(f"[WX] start download image, img_url={img_url}")
            pic_res = requests.get(img_url, stream=True)
            image_storage = io.BytesIO()
            size = 0
            for block in pic_res.iter_content(1024):
                size += len(block)
                image_storage.write(block)
            logger.info(f"[WX] download image success, size={size}, img_url={img_url}")
            image_storage.seek(0)
            if ".webp" in img_url:
                try:
                    image_storage = convert_webp_to_png(image_storage)
                except Exception as e:
                    logger.error(f"Failed to convert image: {e}")
                    return
            itchat.send_image(image_storage, toUserName=receiver)
            logger.info("[WX] sendImage url={}, receiver={}".format(img_url, receiver))
        elif reply.type == ReplyType.IMAGE:  # 从文件读取图片
            image_storage = reply.content
            image_storage.seek(0)
            itchat.send_image(image_storage, toUserName=receiver)
            logger.info("[WX] sendImage, receiver={}".format(receiver))
        elif reply.type == ReplyType.FILE:  # 新增文件回复类型
            file_storage = reply.content
            itchat.send_file(file_storage, toUserName=receiver)
            logger.info("[WX] sendFile, receiver={}".format(receiver))
        elif reply.type == ReplyType.VIDEO:  # 新增视频回复类型
            video_storage = reply.content
            itchat.send_video(video_storage, toUserName=receiver)
            logger.info("[WX] sendFile, receiver={}".format(receiver))
        elif reply.type == ReplyType.VIDEO_URL:  # 新增视频URL回复类型
            video_url = reply.content
            logger.debug(f"[WX] start download video, video_url={video_url}")
            video_res = requests.get(video_url, stream=True)
            video_storage = io.BytesIO()
            size = 0
            for block in video_res.iter_content(1024):
                size += len(block)
                video_storage.write(block)
            logger.info(f"[WX] download video success, size={size}, video_url={video_url}")
            video_storage.seek(0)
            itchat.send_video(video_storage, toUserName=receiver)
            logger.info("[WX] sendVideo url={}, receiver={}".format(video_url, receiver))


def _send_login_success():
    try:
        from common.linkai_client import chat_client
        if chat_client.client_id:
            chat_client.send_login_success()
    except Exception as e:
        pass


def _send_logout():
    try:
        from common.linkai_client import chat_client
        if chat_client.client_id:
            chat_client.send_logout()
    except Exception as e:
        pass


def _send_qr_code(qrcode_list: list):
    try:
        from common.linkai_client import chat_client
        if chat_client.client_id:
            chat_client.send_qrcode(qrcode_list)
    except Exception as e:
        pass



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : common
# @Time         : 2023/12/4 16:32
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  :

from meutils.pipe import *
from meutils.decorators.retry import retrying
from urllib.parse import quote


@retrying
@lru_cache(maxsize=1024)
def shorten_url(url, shortener='dagd'):
    """
        https://w3.do/k0xi_szO
        https://da.gd/haFET # 耗时更短
        https://tinyurl.com/ym8xkpyl
        https://clck.ru/36vUv9 # 微信没法直接跳转

    :param url:
    :param shortener:
    :return:
    """
    if shortener.startswith('w3'):
        url = f"https://w3.do/get?url={url}"
        url = f"""https://{requests.get(url).json().get("url")}"""

    elif shortener.startswith('ft'):  # https://www.ft12.com/
        API_KEY = "18550288233@81fbd6ad8469aba2b45e26929d4ff6a8"

        url = f"http://two.ft12.com/api.php?format=json&url={quote(url)}&apikey={API_KEY}"
        url = f"""{requests.get(url).json().get("url")}"""


    else:
        from pyshorteners import Shortener

        url = Shortener().__getattr__(shortener).short(url)
    return url


@retrying
@lru_cache(maxsize=1024)
def data2qrcodeurl(data, qrcode_api: str = "https://api.isoyu.com/qr/?m=1&e=L&p=20&url={}", shortened: bool = False):
    """
        apis = [
            "https://api.isoyu.com/qr/?m=1&e=L&p=8&url={}",
            "https://api.qrserver.com/v1/create-qr-code/?data={}",
            # "https://api.qrserver.com/v1/create-qr-code/?size=500×500&data={}",
            "https://my.tv.sohu.com/user/a/wvideo/getQRCode.do?text={}"
        ]

        for api in apis:
            print(to_qrcode(url, api))

    :param url:
    :param qrcode_api:
    :return: https://api.isoyu.com/qr/?m=1&e=L&p=20&url=https://vip.chatllm.vip/
    """
    url = qrcode_api.format(data)

    # url = qrcode_api.format(quote(data))

    return shorten_url(url, 'w3') if shortened else url


@retrying
@lru_cache
def qrcodeurl2data(qrcodeurl):
    """

    :param qrcodeurl: https://api.isoyu.com/qr/?m=1&e=L&p=8&url=123
    :return:
    """
    headers = {
        'User-Agent': 'Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) '
                      'Chrome/63.0.3239.132 Safari/537.36 QIHU 360SE '
    }
    return requests.get(qrcodeurl, headers=headers).content


if __name__ == '__main__':
    # url = "https://vip.chatllm.vip/"
    # url = "http://www.zhihechat.com/#home"

    "https://nextchat.chatfire.cn/#/?settings={\"key\":\"{key}\"}"
    url = """https://ichat.chatllm.vip/#/?settings={"key":"sk-"}"""
    # url = "http://111.173.117.175:40000/rag/"
    url = "http://oss.cloudbellai.com/file/%E5%8A%9F%E8%83%BD%E7%82%B9%E5%9B%BE%E7%89%87/MES%E8%A7%A3%E5%86%B3%E6%96%B9%E6%A1%88/%E3%80%90%E6%8E%A8%E8%8D%90%E3%80%91MES%E7%B3%BB%E7%BB%9F%E8%A7%84%E5%88%92%E4%B8%8E%E5%AE%9E%E6%96%BD/MES%E7%B3%BB%E7%BB%9F%E8%BE%B9%E7%95%8C%E4%B8%8E%E9%9B%86%E6%88%90.png"
    # print(shorten_url(url, 'w3'))
    # print(shorten_url(url, 'dagd'))
    # print(shorten_url(url, 'clckru'))
    # print(shorten_url(url, 'tinyurl'))
    # print(shorten_url(url, 'ft'))
    print(shorten_url(url))

    from meutils.io.image import image_to_base64

    # apis = [
    #     # "https://api.isoyu.com/qr/?m=1&e=L&p=8&url={}",
    #     # "https://my.tv.sohu.com/user/a/wvideo/getQRCode.do?text={}",
    #     # "https://api.qrserver.com/v1/create-qr-code/?data={}",
    #     "https://api.qrserver.com/v1/create-qr-code/?size=500×500&data={}",
    # ]
    #
    # for api in apis:
        # print(data2qrcodeurl('知盒欢迎你', api, shortened=True))
        # print(data2qrcodeurl("http://www.zhihechat.com/#home", api, shortened=True))
        # print(data2qrcodeurl("https://www.baidu.com", api, shortened=True))

        # print(data2qrcodeurl(image_to_base64('qr.png'), api, shortened=True))

        # break

    # print(qrcodeurl2data('https://api.isoyu.com/qr/?m=1&e=L&p=8&url=123'))





#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : tql-ANN.
# @File         : ANN
# @Time         : 2019-12-04 20:05
# @Author       : yuanjie
# @Email        : meutils@qq.com
# @Software     : PyCharm
# @Description  : 尽量统一接口
# 从内存
# 从文件
# todo: 增加idmapping并提供在线服务


import gensim


class ANN(object):

    def __init__(self, vector_size=768):
        self.model = gensim.models.KeyedVectors(vector_size)

    def add_vec(self, vectors, ids=None, replace=False):
        """todo 多次加载好像有问题"""
        if ids is None:
            ids = range(len(vectors))
        self.model.add_vectors(ids, vectors, replace=replace)
        # self.model.fill_norms(force=True)  # 归一化
        return self


if __name__ == '__main__':
    import numpy as np

    ann = ANN()
    vectors = np.random.random((10, 768))

    ann.add_vec(vectors)
    _ = ann.model.similar_by_vector(vectors[0])
    print(_)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : products
# @Time         : 2025/8/13 17:01
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  : 

from meutils.pipe import *
from meutils.llm.clients import AsyncClient
"""
product-2c8b72 
4.5v

"""

async def create_pre_order(api_key, product_id: str = "product-2c8b72"):
    payload = {"channelCode": "BALANCE", "isMobile": False, "num": 1, "payPrice": 0, "productId": product_id}
    client = AsyncClient(base_url="https://bigmodel.cn", api_key=api_key)
    response = await client.post("/api/biz/product/createPreOrder", body=payload, cast_to=object)
    return response


if __name__ == '__main__':
    arun(create_pre_order("9df724995f384c2e91d673864d1d32eb.aeLMBoocPyRfGBx8"))





#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : MeUtils.
# @File         : notice
# @Time         : 2021/4/2 3:46 下午
# @Author       : yuanjie
# @WeChat       : 313303303
# @Software     : PyCharm
# @Description  : 
"""注意事项
1. wrapped(*args, **kwargs)不要重复执行


todo:
feishu keras callback: 训练日志通知

"""

from meutils.pipe import *


def feishu_hook(title='Task Done', text=None, hook_url=None):
    """装饰器里不可变参数

    :param title:
    :param text: 如果为空，用函数返回值填充
    :param hook_url: hook_url或者群名称
    :return:
    """
    hook_url = hook_url or "https://open.feishu.cn/open-apis/bot/v2/hook/f7cf6f2a-30da-4e7a-ae6f-b48c8bb1ecf8"

    @wrapt.decorator
    def wrapper(wrapped, instance, args, kwargs):
        s = time.time()
        r = wrapped(*args, **kwargs)
        e = time.time()

        mins = (e - s) // 60

        logger.info(f"{title} done in {mins} m")

        if text is None:  # text没法直接赋值
            body = {"title": title, "text": str(r) + f"\n耗时 {mins} m"}
        else:
            body = {"title": title, "text": text + f"\n耗时 {mins} m"}

        requests.post(hook_url, json=body).json()

        return r

    return wrapper



if __name__ == '__main__':
    @feishu_hook('catch hook')
    # @feishu_catch()
    def f():
        # 1 / 0
        print(time.time())
        # return 'RES'


    mm = {
        "msg_type": "interactive",
        "card": {
            "elements": [
                {
                    "tag": "div",
                    "text": {
                        "content": "**西湖**，位于浙江省杭州市西湖区龙井路1号，杭州市区西部，景区总面积49平方千米，汇水面积为21.22平方千米，湖面面积为6.38平方千米。",
                        "tag": "lark_md"
                    }
                },
                # {
                #     "actions": [{
                #         "tag": "button",
                #         "text": {
                #             "content": "更多景点介绍 :玫瑰:",
                #             "tag": "lark_md"
                #         },
                #         "url": "https://www.example.com",
                #         "type": "default",
                #         "value": {}
                #     }],
                #     "tag": "action"
                # }
            ],
            "header": {
                "title": {
                    "content": "今日旅游推荐",
                    "tag": "plain_text"
                }
            }
        }
    }


    class Text(BaseModel):
        content: str
        tag: str = 'lark_md'


    class Element(BaseModel):
        text: Text
        tag: str = 'div'


    class Header(BaseModel):
        title: Text


    class Card(BaseModel):
        header: Header

        elements: List[Element]


    class Message(BaseModel):
        """模拟 https://open.feishu.cn/tool/cardbuilder?from=apireference&templateId=ctp_AAmj6jORJix1"""
        msg_type: str = 'interactive'
        card: Card



    m = Message(
        card=Card(
            header=Header(title=Text(content='这是一个标题')),
            elements=[
                Element(text=Text(content='**南京**')),
                # Element(text=Text(content="at所有人<at id=all></at> \n 换行消息")),
                Element(text=Text(content="source")),

            ]
        )
    )

    url = "https://open.feishu.cn/open-apis/bot/v2/hook/f7cf6f2a-30da-4e7a-ae6f-b48c8bb1ecf8"
    r = requests.post(url, json=mm)
    print(r.text)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : logger
# @Time         : 2023/8/21 09:01
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  : https://pawamoy.github.io/posts/unify-logging-for-a-gunicorn-uvicorn-app/

from fastapi import Request, Response
from fastapi.routing import APIRoute
from gunicorn.glogging import Logger

from starlette.background import BackgroundTask
from starlette.responses import Response, StreamingResponse

from meutils.pipe import *
from loguru import logger


####################################################################################
class LoggerSettings(BaseModel):
    sink: Any = os.getenv("LOG_SINK")  # todo: 邮件 飞书等等

    level: str = logging.getLevelName(os.getenv("LOG_LEVEL", "DEBUG"))
    serialize: bool = True if os.getenv("LOG_SERIALIZE", '0') != '0' else False
    rotation: str = os.getenv("LOG_ROTATION", '00:00')
    retention: str = os.getenv("LOG_RETENTION", '7 days')


settings = LoggerSettings()


####################################################################################

def init_logger() -> None:
    logging.root.handlers = [InterceptHandler()]
    logging.root.setLevel(settings.level)

    # Remove all log handlers and propagate to root logger
    for name in logging.root.manager.loggerDict.keys():
        # Do not remove gunicorn loggers
        if "gunicorn" not in name:
            logging.getLogger(name).handlers = []
            logging.getLogger(name).propagate = True

    # Configure sqlalchemy engine logger, which by default is set to WARNING level
    logging.getLogger("sqlalchemy.engine").setLevel(settings.level)

    # Configure logger (again) if gunicorn is not used
    handlers = [{"sink": sys.stdout}]
    if settings.sink:  # 覆盖
        handlers = [
            {
                "sink": settings.sink,
                "level": settings.level,
                "serialize": settings.serialize,
                "enqueue": True,
                "rotation": settings.rotation,
                "retention": settings.retention
            }
        ]
    logger.configure(handlers=handlers)


class InterceptHandler(logging.Handler):
    def emit(self, record):
        # get corresponding Loguru level if it exists
        try:
            level = logger.level(record.levelname).name
        except ValueError:
            level = record.levelno

        # find caller from where originated the logged message
        frame, depth = sys._getframe(6), 6
        while frame and frame.f_code.co_filename == logging.__file__:
            frame = frame.f_back
            depth += 1

        logger.opt(depth=depth, exception=record.exc_info).log(level, record.getMessage())


class GunicornLogger(Logger):
    def setup(self, cfg) -> None:
        handler = InterceptHandler()

        # Add log handler to logger and set log level
        self.error_log.addHandler(handler)
        self.error_log.setLevel(settings.level)
        self.access_log.addHandler(handler)
        self.access_log.setLevel(settings.level)

        # Configure logger before gunicorn starts logging
        handlers = [{"sink": sys.stdout, "level": settings.level, "serialize": settings.serialize}]
        if settings.sink:  # 覆盖
            handlers = [
                {
                    "sink": settings.sink,
                    "level": settings.level,
                    "serialize": settings.serialize,
                    "enqueue": True,
                    "rotation": settings.rotation,
                    "retention": settings.retention
                }
            ]
        logger.configure(handlers=handlers)


class LoggingRoute(APIRoute):
    """
    # Use this for
    # router = APIRouter(route_class=LoggingRoute)
    # when you want to split up your routes and use include_router()
    # https://fastapi.tiangolo.com/advanced/custom-request-and-route/#custom-apiroute-class-in-a-router
    """

    def get_route_handler(self) -> Callable:
        original_route_handler = super().get_route_handler()

        async def custom_route_handler(request: Request) -> Response:
            id = str(uuid.uuid4())
            req_body = await request.body()
            req_info = {
                "request": {
                    "id": id,
                    "method": request.method,
                    "path": request.url.path,
                    "ip": request.client.host,
                    "body": req_body
                }
            }

            before = time.perf_counter()
            response = await original_route_handler(request)
            duration = (time.perf_counter() - before) * 1000
            duration = f"{duration:0.2f}ms"
            response.headers["X-Response-Time"] = duration

            res_info = {
                "response": {
                    "id": id,
                    "status": "successful" if response.status_code < 400 else "failed",
                    "status_code": response.status_code,
                    "duration": duration,
                }
            }

            if isinstance(response, StreamingResponse):  # todo: body_iterator
                res_body = b''
                async for item in response.body_iterator:
                    res_body += item
                res_info["response"]["body"] = res_body

                task = BackgroundTask(self.log_request_response_info, req_info, res_info)

                return Response(
                    content=res_body,
                    status_code=response.status_code,
                    headers=dict(response.headers),
                    media_type=response.media_type,
                    background=task
                )
            else:
                res_body = response.body
                res_info["response"]["body"] = res_body
                response.background = BackgroundTask(self.log_request_response_info, req_info, res_info)

                return response

        return custom_route_handler

    def log_request_response_info(self, req_info, res_info):
        logger.info(req_info)
        logger.info(res_info)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : volc
# @Time         : 2024/12/31 13:20
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  : 


import hashlib
import hmac
import os
from urllib.parse import quote

from meutils.pipe import *

# 以下参数视服务不同而不同，一个服务内通常是一致的
Service = "iam"
Version = "2018-01-01"
Region = "cn-beijing"
Host = "iam.volcengineapi.com"
ContentType = "application/x-www-form-urlencoded"

# 请求的凭证，从IAM或者STS服务中获取
AK = os.getenv("ARK_ACCESS_KEY")
SK = os.getenv("ARK_SECRET_ACCESS_KEY")
# 当使用临时凭证时，需要使用到SessionToken传入Header，并计算进SignedHeader中，请自行在header参数中添加X-Security-Token头
# SessionToken = ""


def norm_query(params):
    query = ""
    for key in sorted(params.keys()):
        if type(params[key]) == list:
            for k in params[key]:
                query = (
                        query + quote(key, safe="-_.~") + "=" + quote(k, safe="-_.~") + "&"
                )
        else:
            query = (query + quote(key, safe="-_.~") + "=" + quote(params[key], safe="-_.~") + "&")
    query = query[:-1]
    return query.replace("+", "%20")


# 第一步：准备辅助函数。
# sha256 非对称加密
def hmac_sha256(key: bytes, content: str):
    return hmac.new(key, content.encode("utf-8"), hashlib.sha256).digest()


# sha256 hash算法
def hash_sha256(content: str):
    return hashlib.sha256(content.encode("utf-8")).hexdigest()


# 第二步：签名请求函数
def request(method, date, query, header, ak, sk, action, body):
    # 第三步：创建身份证明。其中的 Service 和 Region 字段是固定的。ak 和 sk 分别代表
    # AccessKeyID 和 SecretAccessKey。同时需要初始化签名结构体。一些签名计算时需要的属性也在这里处理。
    # 初始化身份证明结构体
    credential = {
        "access_key_id": ak,
        "secret_access_key": sk,
        "service": Service,
        "region": Region,
    }
    # 初始化签名结构体
    request_param = {
        "body": body,
        "host": Host,
        "path": "/",
        "method": method,
        "content_type": ContentType,
        "date": date,
        "query": {"Action": action, "Version": Version, **query},
    }
    if body is None:
        request_param["body"] = ""
    # 第四步：接下来开始计算签名。在计算签名前，先准备好用于接收签算结果的 signResult 变量，并设置一些参数。
    # 初始化签名结果的结构体
    x_date = request_param["date"].strftime("%Y%m%dT%H%M%SZ")
    short_x_date = x_date[:8]
    x_content_sha256 = hash_sha256(request_param["body"])
    sign_result = {
        "Host": request_param["host"],
        "X-Content-Sha256": x_content_sha256,
        "X-Date": x_date,
        "Content-Type": request_param["content_type"],
    }
    # 第五步：计算 Signature 签名。
    signed_headers_str = ";".join(
        ["content-type", "host", "x-content-sha256", "x-date"]
    )
    # signed_headers_str = signed_headers_str + ";x-security-token"
    canonical_request_str = "\n".join(
        [request_param["method"].upper(),
         request_param["path"],
         norm_query(request_param["query"]),
         "\n".join(
             [
                 "content-type:" + request_param["content_type"],
                 "host:" + request_param["host"],
                 "x-content-sha256:" + x_content_sha256,
                 "x-date:" + x_date,
             ]
         ),
         "",
         signed_headers_str,
         x_content_sha256,
         ]
    )

    # 打印正规化的请求用于调试比对
    print(canonical_request_str)
    hashed_canonical_request = hash_sha256(canonical_request_str)

    # 打印hash值用于调试比对
    print(hashed_canonical_request)
    credential_scope = "/".join([short_x_date, credential["region"], credential["service"], "request"])
    string_to_sign = "\n".join(["HMAC-SHA256", x_date, credential_scope, hashed_canonical_request])

    # 打印最终计算的签名字符串用于调试比对
    print(string_to_sign)
    k_date = hmac_sha256(credential["secret_access_key"].encode("utf-8"), short_x_date)
    k_region = hmac_sha256(k_date, credential["region"])
    k_service = hmac_sha256(k_region, credential["service"])
    k_signing = hmac_sha256(k_service, "request")
    signature = hmac_sha256(k_signing, string_to_sign).hex()

    sign_result["Authorization"] = "HMAC-SHA256 Credential={}, SignedHeaders={}, Signature={}".format(
        credential["access_key_id"] + "/" + credential_scope,
        signed_headers_str,
        signature,
    )
    header = {**header, **sign_result}
    # header = {**header, **{"X-Security-Token": SessionToken}}
    # 第六步：将 Signature 签名写入 HTTP Header 中，并发送 HTTP 请求。
    r = requests.request(method=method,
                         url="https://{}{}".format(request_param["host"], request_param["path"]),
                         headers=header,
                         params=request_param["query"],
                         data=request_param["body"],
                         )
    return r.json()


if __name__ == "__main__":
    # response_body = request("Get", datetime.datetime.utcnow(), {}, {}, AK, SK, "ListUsers", None)
    # print(response_body)

    now = datetime.datetime.utcnow()

    # Body的格式需要配合Content-Type，API使用的类型请阅读具体的官方文档，如:json格式需要json.dumps(obj)
    # response_body = request("GET", now, {"Limit": "2"}, {}, AK, SK, "ListUsers", None)
    # print(response_body)

    response_body = request("POST", now, {"Limit": "10"}, {}, AK, SK, "ListUsers", "UnUseParam=ASDF")
    print(response_body)


#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : Python.
# @File         : mimongo
# @Time         : 2020-03-20 11:26
# @Author       : yuanjie
# @Email        : meutils@qq.com
# @Software     : PyCharm
# @Description  :
# https://www.jb51.net/article/159652.htm
# https://www.cnblogs.com/kaituorensheng/p/5181410.html

from meutils.pipe import *
from meutils.decorators import singleton
from pymongo import MongoClient, ReadPreference


class DBConf(BaseConfig):
    database: str
    user: str
    passwd: str
    ips: str
    dns: str
    replicaSet: str


@singleton
class Mongo(object):

    def __init__(self, db='mig3_algo_push', url="mongodb://localhost:27017", only_read=True):
        """
        :param db:
        :param print_info:
        """
        if url is None:
            conf = DBConf.parse_zk('/push/db/mongodb')
            url = f"mongodb://{conf.user}:{conf.passwd}@{conf.dns}/{conf.database}?replicaSet={conf.replicaSet}&authSource=admin"

        self.client = MongoClient(url)
        if only_read:
            self.db = self.client.get_database(db, read_preference=ReadPreference.SECONDARY_PREFERRED)
        else:
            self.db = self.client[db]

        self.client.get_database()

        self.client_info = {
            "主节点": self.client.is_primary,
            "最大连接数": self.client.max_pool_size,
            'self.client.admin.command': self.client.admin.command('ismaster')
        }

    def starter(self):
        """
        # info
        collection.count_documents({})

        # 增
        collection.insert_one({'x': 1})
        collection.insert_one({'xx': 2})
        ids = collection.insert_many([{'xxx': 3}, {'xxx': 4}])

        from bson.objectid import ObjectId

        collection.find_one({'x': 1})
        collection.find_one({'_id': ObjectId('5e743cea06be472ac7298def')})

        # 复杂的查询
        list(collection.find({})) # collection.find().count()
        list(collection.find({'xx': {'$gt': 1}}))

        condition = {'s': 1}
        # update_one update_many
        # result = collection.replace_one(condition, {'s': 1, 'ss': 2}) # 完全替换 # upsert=True强拆
        result = collection.replace_one(condition, {'$set': {'s':1, 'sss': 3}}) # {**d1, **d2}
        """
        pass


if __name__ == '__main__':
    Mongo()



#!/usr/bin/env python

"""The setup script."""
import time
from setuptools import setup, find_packages
# from {{cookiecutter.project_slug}} import __version__

version = time.strftime("%Y.%m.%d.%H.%M.%S", time.localtime())

with open('README.md') as readme_file:
    readme = readme_file.read()

with open('HISTORY.rst') as history_file:
    history = history_file.read()

with open('requirements.txt', encoding='utf-8') as f:
    requirements = f.read().split('\n')


{%- set license_classifiers = {
    'MIT license': 'License :: OSI Approved :: MIT License',
    'BSD license': 'License :: OSI Approved :: BSD License',
    'ISC license': 'License :: OSI Approved :: ISC License (ISCL)',
    'Apache Software License 2.0': 'License :: OSI Approved :: Apache Software License',
    'GNU General Public License v3': 'License :: OSI Approved :: GNU General Public License v3 (GPLv3)'
} %}

setup(
    author="{{ cookiecutter.full_name.replace('\"', '\\\"') }}",
    author_email='{{ cookiecutter.email }}',
    python_requires='>=3.6',
    classifiers=[
        'Development Status :: 2 - Pre-Alpha',
        'Intended Audience :: Developers',
{%- if cookiecutter.open_source_license in license_classifiers %}
        '{{ license_classifiers[cookiecutter.open_source_license] }}',
{%- endif %}
        'Natural Language :: English',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
    ],
    description="{{ cookiecutter.project_short_description }}",
    {%- if 'no' not in cookiecutter.command_line_interface|lower %}
    entry_points={
        'console_scripts': [
            '{{cookiecutter.project_slug}}={{cookiecutter.project_slug}}.clis.cli:cli'
        ],
    },
    {%- endif %}
    install_requires=requirements,
{%- if cookiecutter.open_source_license in license_classifiers %}
    license="{{ cookiecutter.open_source_license }}",
{%- endif %}
    long_description=readme + '\n\n' + history,
    long_description_content_type="text/markdown",
    include_package_data=True,
    keywords='{{ cookiecutter.project_slug }}',
    name='{{ cookiecutter.project_slug }}',
    packages=find_packages(include=['{{ cookiecutter.project_slug }}', '{{ cookiecutter.project_slug }}.*']),

    test_suite='tests',
    url='https://github.com/{{ cookiecutter.github_username }}/{{ cookiecutter.project_slug }}',
    version=version, # '{{cookiecutter.version}}',
    zip_safe=False,
)




import re


class SplitSentence(object):
    """ 分句，将文本切分为若干句子，其中处理引号的部分逻辑情况较多

    Args:
        text(str): 字符串文本
        criterion(coarse/fine): 句子切分粒度，粗细两种 `coarse` 或 `fine`，
            `coarse` 指的是按句号级别切分，`fine` 指按所有标点符合切分，
            默认按照粗粒度进行切分

    Returns:
        list(str): 分句后的句子列表

    Examples:
        text = '中华古汉语，泱泱大国，历史传承的瑰宝。'
        split_sentence = SplitSentence()

        print(split_sentence(text, criterion='fine'))

        # ['中华古汉语，', '泱泱大国，', '历史传承的瑰宝。']

    """

    def __init__(self):
        self.puncs_fine = None

    def _prepare(self):
        self.puncs_fine = {'……', '\r\n', '，', '。', ';', '；', '…', '！', '!',
                           '?', '？', '\r', '\n', '“', '”', '‘', '’', '：'}
        self.puncs_coarse = {'。', '！', '？', '\n', '“', '”', '‘', '’'}
        self.front_quote_list = {'“', '‘'}
        self.back_quote_list = {'”', '’'}

        self.puncs_coarse_ptn = re.compile('([。“”！？\n])')
        self.puncs_fine_ptn = re.compile('([，：。;“”；…！!?？\r\n])')

    def __call__(self, text, criterion='coarse'):

        if self.puncs_fine is None:
            self._prepare()

        if criterion == 'coarse':
            tmp_list = self.puncs_coarse_ptn.split(text)
        elif criterion == 'fine':
            tmp_list = self.puncs_fine_ptn.split(text)
        else:
            raise ValueError('The parameter `criterion` must be '
                             '`coarse` or `fine`.')

        final_sentences = []
        quote_flag = False

        for sen in tmp_list:
            if sen == '':
                continue

            if criterion == 'coarse':
                if sen in self.puncs_coarse:
                    if len(final_sentences) == 0:  # 即文本起始字符是标点
                        if sen in self.front_quote_list:  # 起始字符是前引号
                            quote_flag = True
                        final_sentences.append(sen)
                        continue

                    # 以下确保当前标点前必然有文本且非空字符串
                    # 前引号较为特殊，其后的一句需要与前引号合并，而不与其前一句合并
                    if sen in self.front_quote_list:
                        if final_sentences[-1][-1] in self.puncs_coarse:
                            # 前引号前有标点如句号，引号等：另起一句，与此合并
                            final_sentences.append(sen)
                        else:
                            # 前引号之前无任何终止标点，与前一句合并
                            final_sentences[-1] = final_sentences[-1] + sen
                        quote_flag = True
                    else:  # 普通,非前引号，则与前一句合并
                        final_sentences[-1] = final_sentences[-1] + sen
                    continue

            elif criterion == 'fine':
                if sen in self.puncs_fine:
                    if len(final_sentences) == 0:  # 即文本起始字符是标点
                        if sen in self.front_quote_list:  # 起始字符是前引号
                            quote_flag = True
                        final_sentences.append(sen)
                        continue

                    # 以下确保当前标点前必然有文本且非空字符串
                    # 前引号较为特殊，其后的一句需要与前引号合并，而不与其前一句合并
                    if sen in self.front_quote_list:
                        if final_sentences[-1][-1] in self.puncs_fine:
                            # 前引号前有标点如句号，引号等：另起一句，与此合并
                            final_sentences.append(sen)
                        else:
                            # 前引号之前无任何终止标点，与前一句合并
                            final_sentences[-1] = final_sentences[-1] + sen
                        quote_flag = True
                    else:  # 普通,非前引号，则与前一句合并
                        final_sentences[-1] = final_sentences[-1] + sen
                    continue

            if len(final_sentences) == 0:  # 起始句且非标点
                final_sentences.append(sen)
                continue

            if quote_flag:  # 当前句子之前有前引号，须与前引号合并
                final_sentences[-1] = final_sentences[-1] + sen
                quote_flag = False
            else:
                if final_sentences[-1][-1] in self.back_quote_list:
                    # 此句之前是后引号，需要考察有无其他终止符，用来判断是否和前句合并
                    if len(final_sentences[-1]) <= 1:
                        # 前句仅一个字符。后引号，则合并
                        final_sentences[-1] = final_sentences[-1] + sen
                    else:  # 前句有多个字符，
                        if criterion == 'fine':
                            if final_sentences[-1][-2] in self.puncs_fine:
                                # 有逗号等，则需要另起一句，该判断不合语文规范，但须考虑此情况
                                final_sentences.append(sen)
                            else:  # 前句无句号，则需要与前句合并
                                final_sentences[-1] = final_sentences[-1] + sen

                        elif criterion == 'coarse':
                            if final_sentences[-1][-2] in self.puncs_coarse:
                                # 有句号，则需要另起一句
                                final_sentences.append(sen)
                            else:  # 前句无句号，则需要与前句合并
                                final_sentences[-1] = final_sentences[-1] + sen
                else:
                    final_sentences.append(sen)

        return final_sentences


split_sentence = SplitSentence()

if __name__ == '__main__':
    text = '央视新闻消息，近日，特朗普老友皮尔斯·摩根喊话特朗普：“美国人的生命比你的选举更重要。如果你继续以自己为中心，继续玩弄愚蠢的政治……如果你意识不到自己>的错误，你就做不对”。目前，特朗普已“取关”了这位老友。'
    res = split_sentence(text, criterion='coarse')
    print(res)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : mi.
# @File         : Neo4j
# @Time         : 2020/9/18 11:26 上午
# @Author       : yuanjie
# @Email        : meutils@qq.com
# @Software     : PyCharm
# @Description  : https://zhuanlan.zhihu.com/p/437824721
# https://blog.csdn.net/juanjuan1314/article/details/90814788

from meutils.pipe import *
from py2neo import Graph, Node, Relationship
from concurrent.futures import ThreadPoolExecutor


class Neo4j(object):
    """TODO
    添加属性值：关系的属性
    """

    def __init__(self, profile="bolt://xx:7687", name='neo4j', password='ai', **settings):
        self.graph = Graph(profile, auth=(name, password), **settings).run
        # self.graph.delete_all()
        self.node_labels = self.graph.schema.node_labels  # 查看图结构中节点标签的类别，返回结果是一个frozenset
        self.relationship_types = self.graph.schema.relationship_types  # 查看图结构中关系的类型
        # graph.schema.get_indexes

    def create_nodes(self, df_nodes, label="Demo", max_workers=30):
        """

        :param label: Node Label 可以理解为一个集合
        :param node_list: [(k, v, r), ] 三元组
        :return:
        """
        df_nodes.columns = ['k', 'v', 'r']
        groups = df_nodes.groupby('k')
        print(f"Num Group: {len(groups)}")

        func = lambda group: [self._create_node(label, nodes) for nodes in tqdm(group[1].values)]

        with ThreadPoolExecutor(max_workers, thread_name_prefix=f"{label}__") as pool:
            _ = pool.map(func, tqdm(groups), chunksize=1)

    def _create_node(self, label, nodes):
        k, v, r = nodes

        node_key = self._create(label, k)
        node_value = self._create(label, v)

        if node_key != node_value:
            node_relation = Relationship(node_key, r, node_value)  # r也可以是节点，
            # node_relation['属性'] = 0
            self.graph.create(node_relation)

    @lru_cache(1024)
    def _create(self, label, name):
        subgraph = Node(label, name=name)  # properties 属性信息
        self.graph.create(subgraph)
        return subgraph


